/*
 * Copyright (C) 2022 European Spallation Source ERIC.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.openepics.names.docker;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.junit.jupiter.api.Assertions.assertNull;
import static org.junit.jupiter.api.Assertions.assertTrue;
import static org.junit.jupiter.api.Assertions.fail;

import java.io.IOException;
import java.net.HttpURLConnection;
import java.util.UUID;

import org.openepics.names.docker.ITUtil.AuthorizationChoice;
import org.openepics.names.docker.ITUtil.EndpointChoice;
import org.openepics.names.rest.beans.NameElement;
import org.openepics.names.rest.beans.Status;
import org.openepics.names.rest.beans.response.ResponseBoolean;
import org.openepics.names.rest.beans.response.ResponseBooleanList;
import org.openepics.names.rest.beans.response.ResponsePageNameElements;
import org.openepics.names.util.EnumUtil.NameChoice;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;

/**
 * Utility class to help (Docker) integration tests for Naming and PostgreSQL.
 *
 * @author Lars Johansson
 */
public class ITUtilNameElement {

    /**
     * This class is not to be instantiated.
     */
    private ITUtilNameElement() {
        throw new IllegalStateException("Utility class");
    }

    // ----------------------------------------------------------------------------------------------------

    /**
     * Assert that array of name elements has expected length.
     *
     * @param actual array of name elements
     * @param expectedLength expected length of array
     */
    static void assertContentLength(NameElement[] actual, int expectedLength) {
        assertNotNull(actual);
        assertEquals(expectedLength, actual.length);
    }

    /**
     * Assert that array of name elements has length one and name element is available.
     *
     * @param actual array of name elements
     * @return name element
     */
    static NameElement assertContentLengthOne(NameElement[] actual) {
        assertContentLength(actual, 1);
        assertNotNull(actual[0]);
        return actual[0];
    }

    /**
     * Assert that response page with list of name elements has expected length.
     *
     * @param actual response page with list of name elements
     * @param expectedLength expected length of list
     */
    static void assertContentLength(ResponsePageNameElements actual, int expectedLength) {
        assertNotNull(actual);
        assertEquals(expectedLength, actual.getList().size());
        assertEquals(expectedLength, actual.getListSize());
    }

    /**
     * Assert that response page with list of name elements has length one and name element is available.
     *
     * @param actual response page with list of name elements
     * @return name element
     */
    static NameElement assertContentLengthOne(ResponsePageNameElements actual) {
        assertContentLength(actual, 1);
        assertNotNull(actual.getList().get(0));
        return actual.getList().get(0);
    }

    // ----------------------------------------------------------------------------------------------------

    /**
     * Assert that arrays are equal with same length and same content in each array position.
     *
     * @param actual actual array of name elements
     * @param expected expected arbitrary number of name elements
     */
    static void assertEqualsNameElements(NameElement[] actual, NameElement... expected) {
        if (expected != null) {
            assertNotNull(actual);
            assertEquals(expected.length, actual.length);
            for (int i=0; i<expected.length; i++) {
                assertTrue(expected[i].equals(actual[i]));
            }
        } else {
            assertNull(actual);
        }
    }

    /**
     * Assert content of name element.
     *
     * @param actual name element
     * @param expectedUuid expected uuid
     * @param expectedParentSystemstructure expected parent system structure uuid (system group, system, subsystem)
     * @param expectedParentDevicestructure parent device structure uuid (device type)
     * @param expectedSystemstructure expected system structure
     * @param expectedDevicestructure expected device structure
     * @param expectedIndex expected index
     * @param expectedName expected name
     * @param expectedDescription expected description
     * @param expectedStatus expected status
     * @param expectedIsLatest expected latest
     * @param expectedIsDeleted expected deleted
     * @param expectedWho expected who
     * @param expectedComment expected comment
     */
    static void assertContent(NameElement actual,
            UUID expectedUuid, UUID expectedParentSystemstructure, UUID expectedParentDevicestructure,
            String expectedSystemstructure, String expectedDevicestructure,
            String expectedIndex, String expectedName,
            String expectedDescription, Status expectedStatus, Boolean expectedIsLatest, Boolean expectedIsDeleted,
            String expectedWho, String expectedComment) {
        assertNotNull(actual);
        assertEquals(expectedUuid, actual.getUuid());
        assertEquals(expectedParentSystemstructure, actual.getParentsystemstructure());
        assertEquals(expectedParentDevicestructure, actual.getParentdevicestructure());
        assertEquals(expectedSystemstructure, actual.getSystemstructure());
        assertEquals(expectedDevicestructure, actual.getDevicestructure());
        assertEquals(expectedIndex, actual.getIndex());
        assertEquals(expectedName, actual.getName());
        assertEquals(expectedDescription, actual.getDescription());
        assertEquals(expectedStatus, actual.getStatus());
        assertEquals(expectedIsLatest, actual.isLatest());
        assertEquals(expectedIsDeleted, actual.isDeleted());
        assertNotNull(actual.getWhen());
        assertEquals(expectedWho, actual.getWho());
        assertEquals(expectedComment, actual.getComment());
    }

    // ----------------------------------------------------------------------------------------------------

    /**
     * Utility method to read name elements and assert number of items in list.
     *
     * @param queryString query string
     * @param expectedLessThanOrEqual (if non-negative number) less than or equal to this number of items
     * @param expectedGreaterThanOrEqual (if non-negative number) greater than or equal to this number of items
     * @return number of items in list
     */
    public static Integer assertRead(String queryString, int expectedLessThanOrEqual, int expectedGreaterThanOrEqual) {
        try {
            ObjectMapper mapper = new ObjectMapper();
            String[] response = null;
            ResponsePageNameElements responsePageNameElements = null;

            response = ITUtil.doGetJson(ITUtil.HTTP_IP_PORT_NAMING_API_V1_NAMES + queryString);
            ITUtil.assertResponseLength2Code(response, HttpURLConnection.HTTP_OK);
            responsePageNameElements = mapper.readValue(response[1], ResponsePageNameElements.class);
            if (expectedLessThanOrEqual >= 0) {
                assertTrue(responsePageNameElements.getListSize() <= expectedLessThanOrEqual);
            }
            if (expectedGreaterThanOrEqual >= 0) {
                assertTrue(responsePageNameElements.getListSize() >= expectedGreaterThanOrEqual);
            }

            return responsePageNameElements.getListSize();
        } catch (IOException e) {
            fail();
        } catch (Exception e) {
            fail();
        }
        return null;
    }

    /**
     * Utility method to get name equivalence of a mnemonic or name.
     *
     * @param name name
     * @param expected expected response
     */
    public static void assertEquivalence(String name, String expected) {
        try {
            String[] response = ITUtil.doGetJson(ITUtil.HTTP_IP_PORT_NAMING_API_V1_NAMES + "/equivalence/" + name);
            ITUtil.assertResponseLength2CodeOKContent(response, expected);
        } catch (IOException e) {
            fail();
        } catch (Exception e) {
            fail();
        }
    }

    /**
     * Utility method to check if name exists.
     *
     * @param name name
     * @param expected expected response
     */
    public static void assertExists(String name, Boolean expected) {
        assertExists(name, expected, null);
    }

    /**
     * Utility method to check if name exists.
     *
     * @param name name
     * @param expected expected response
     * @param expectedMessageEmpty expected message empty
     */
    public static void assertExists(String name, Boolean expected, Boolean expectedMessageEmpty) {
        try {
            ObjectMapper mapper = new ObjectMapper();
            String[] response = null;

            response = ITUtil.doGetJson(ITUtil.HTTP_IP_PORT_NAMING_API_V1_NAMES + "/exists/" + name);
            ITUtil.assertResponseLength2CodeOK(response);
            if (expectedMessageEmpty != null) {
                ITUtil.assertEqualsResponseBoolean(mapper.readValue(response[1], ResponseBoolean.class), expected, expectedMessageEmpty);
            } else {
                ITUtil.assertEqualsResponseBoolean(mapper.readValue(response[1], ResponseBoolean.class), expected);
            }
        } catch (IOException e) {
            fail();
        } catch (Exception e) {
            fail();
        }
    }

    /**
     * Utility method to check if name is legacy name.
     *
     * @param name name
     * @param expected expected response
     */
    public static void assertIsLegacy(String name, Boolean expected) {
        try {
            ObjectMapper mapper = new ObjectMapper();
            String[] response = null;

            response = ITUtil.doGetJson(ITUtil.HTTP_IP_PORT_NAMING_API_V1_NAMES + "/islegacy/" + name);
            ITUtil.assertResponseLength2CodeOK(response);
            ITUtil.assertEqualsResponseBoolean(mapper.readValue(response[1], ResponseBoolean.class), expected);
        } catch (IOException e) {
            fail();
        } catch (Exception e) {
            fail();
        }
    }

    /**
     * Utility method to check if name is valid to create.
     *
     * @param name name
     * @param expected expected response
     */
    public static void assertIsValidToCreate(String name, Boolean expected) {
        try {
            ObjectMapper mapper = new ObjectMapper();
            String[] response = null;

            response = ITUtil.doGetJson(ITUtil.HTTP_IP_PORT_NAMING_API_V1_NAMES + "/isvalidtocreate/" + name);
            ITUtil.assertResponseLength2CodeOK(response);
            ITUtil.assertEqualsResponseBoolean(mapper.readValue(response[1], ResponseBoolean.class), expected);
        } catch (IOException e) {
            fail();
        } catch (Exception e) {
            fail();
        }
    }

    // ----------------------------------------------------------------------------------------------------

    /**
     * Utility method to (try to) validate a name element from given json and assert expected response.
     * Method expected to NOT succeed to validate
     * but instead assert response code (possibly also a non-empty message).
     *
     * @param json json
     * @param nameChoice name choice
     * @param responseCode response code
     */
    public static void assertValidate(String json, NameChoice nameChoice, int responseCode) {
        assertValidate(json, nameChoice, responseCode, Boolean.FALSE, Boolean.FALSE);
    }

    public static void assertValidate(String json, NameChoice nameChoice, int responseCode, Boolean expected, Boolean expectedMessageEmpty) {
        String validatePath = getValidatePath(nameChoice);

        try {
            ObjectMapper mapper = new ObjectMapper();
            String[] response = null;

            response = ITUtil.runShellCommand(ITUtil.curlGetPathJson(AuthorizationChoice.NONE, EndpointChoice.NAMES, validatePath, json));
            ITUtil.assertResponseLength2Code(response, responseCode);
            if (response[1] != null) {
                ITUtil.assertEqualsResponseBoolean(mapper.readValue(response[1], ResponseBooleanList.class), expected, expectedMessageEmpty);
            }
        } catch (IOException e) {
            fail();
        } catch (InterruptedException e) {
            fail();
        } catch (Exception e) {
            fail();
        }
    }

    /**
     * Utility method to set name element attributes name and index and then validate create and assert expected response.
     *
     * @param nameElement name element
     * @param nameChoice name choice
     * @param name name
     * @param index index
     * @param expected expected response
     */
    public static void assertValidate(NameElement nameElement, NameChoice nameChoice, String name, String index, Boolean expected) {
        nameElement.setNameAndIndex(name, index);
        assertValidate(nameElement, nameChoice, expected);
    }

    /**
     * Utility method to validate and assert expected response.
     *
     * @param nameElement name element
     * @param nameChoice name choice
     * @param expected expected response
     */
    public static void assertValidate(NameElement nameElement, NameChoice nameChoice, Boolean expected) {
        String validatePath = getValidatePath(nameChoice);

        try {
            ObjectMapper mapper = new ObjectMapper();
            String[] response = null;

            response = ITUtil.runShellCommand(ITUtil.curlGetPathJson(AuthorizationChoice.NONE, EndpointChoice.NAMES, validatePath, "[" + mapper.writeValueAsString(nameElement) + "]"));
            ITUtil.assertResponseLength2Code(response, HttpURLConnection.HTTP_OK);
            ITUtil.assertEqualsResponseBoolean(mapper.readValue(response[1], ResponseBooleanList.class), expected);
        } catch (IOException e) {
            fail();
        } catch (InterruptedException e) {
            fail();
        } catch (Exception e) {
            fail();
        }
    }

    /**
     * Utility method to return validate path for name choice.
     *
     * @param nameChoice name choice
     * @return validate path
     */
    private static String getValidatePath(NameChoice nameChoice) {
        String validatePath = "";
        switch (nameChoice) {
            case CREATE:
                validatePath = "/validatecreate";
                break;
            case UPDATE:
                validatePath = "/validateupdate";
                break;
            case DELETE:
                validatePath = "/validatedelete";
                break;
            default:
                break;
        }
        return validatePath;
    }

    // ----------------------------------------------------------------------------------------------------

    /**
     * Utility method to (try to) create a name element and assert result response code.
     * Method expected to NOT succeed to create a name element
     * but instead assert response code (possibly also a non-empty message).
     *
     * @param nameElement name element
     * @param responseCode response code
     * @throws JsonProcessingException
     */
    public static void assertCreate(NameElement nameElement, int responseCode) throws JsonProcessingException {
        ObjectMapper mapper = new ObjectMapper();
        assertCreate("[" + mapper.writeValueAsString(nameElement) + "]", responseCode);
    }

    /**
     * Utility method to (try to) create a name element from given json and assert result response code.
     * Method expected to NOT succeed to create a name element
     * but instead assert response code (possibly also a non-empty message).
     *
     * @param json json
     * @param responseCode response code
     */
    public static void assertCreate(String json, int responseCode) {
        try {
            String[] response = ITUtil.runShellCommand(ITUtil.curlPostPathJson(AuthorizationChoice.NONE, EndpointChoice.NAMES, "", json));
            ITUtil.assertResponseLength2Code(response, responseCode);
            // ITUtil.assertMessageNotEmpty(mapper.readValue(response[1], Response.class));
        } catch (IOException e) {
            fail();
        } catch (InterruptedException e) {
            fail();
        } catch (Exception e) {
            fail();
        }
    }

    /**
     * Utility method to create a name element and assert result.
     *
     * @param nameElement name element
     * @return created name element
     */
    static NameElement assertCreate(NameElement nameElement) {
        try {
            ObjectMapper mapper = new ObjectMapper();
            String[] response = null;
            UUID uuid = null;

            NameElement[] createdNameElements = null;
            NameElement   createdNameElement  = null;

            response = ITUtil.runShellCommand(ITUtil.curlGetPathJson(AuthorizationChoice.NONE, EndpointChoice.NAMES, "/validatecreate", "[" + mapper.writeValueAsString(nameElement) + "]"));
            ITUtil.assertResponseLength2Code(response, HttpURLConnection.HTTP_OK);
            ITUtil.assertEqualsResponseBoolean(mapper.readValue(response[1], ResponseBooleanList.class), Boolean.TRUE);

            response = ITUtil.runShellCommand(ITUtil.curlPostPathJson(AuthorizationChoice.NONE, EndpointChoice.NAMES, "", "[" + mapper.writeValueAsString(nameElement) + "]"));
            ITUtil.assertResponseLength2Code(response, HttpURLConnection.HTTP_OK);
            createdNameElements = mapper.readValue(response[1], NameElement[].class);
            createdNameElement = ITUtilNameElement.assertContentLengthOne(createdNameElements);
            assertNotNull(createdNameElement.getUuid());

            uuid = createdNameElement.getUuid();
            nameElement.setUuid(uuid);

            ITUtilNameElement.assertContent(createdNameElement,
                    uuid, nameElement.getParentsystemstructure(), nameElement.getParentdevicestructure(),
                    nameElement.getSystemstructure(), nameElement.getDevicestructure(),
                    nameElement.getIndex(), nameElement.getName(),
                    nameElement.getDescription(), nameElement.getStatus(), Boolean.TRUE, Boolean.FALSE,
                    "test who", nameElement.getComment());
            assertNotNull(createdNameElement.getWhen());

            return createdNameElement;
        } catch (IOException e) {
            fail();
        } catch (InterruptedException e) {
            fail();
        } catch (Exception e) {
            fail();
        }

        return null;
    }

    /**
     * Utility method to update a name element and assert result.
     *
     * @param nameElement name element
     * @return updated name element
     */
    static NameElement assertUpdate(NameElement nameElement) {
        try {
            ObjectMapper mapper = new ObjectMapper();
            String[] response = null;
            UUID uuid = null;

            NameElement[] updatedNameElements = null;
            NameElement   updatedNameElement  = null;

            response = ITUtil.runShellCommand(ITUtil.curlGetPathJson(AuthorizationChoice.NONE, EndpointChoice.NAMES, "/validateupdate", "[" + mapper.writeValueAsString(nameElement) + "]"));
            ITUtil.assertResponseLength2Code(response, HttpURLConnection.HTTP_OK);
            ITUtil.assertEqualsResponseBoolean(mapper.readValue(response[1], ResponseBooleanList.class), Boolean.TRUE);

            response = ITUtil.runShellCommand(ITUtil.curlPutPathJson(AuthorizationChoice.NONE, EndpointChoice.NAMES, "", "[" + mapper.writeValueAsString(nameElement) + "]"));
            ITUtil.assertResponseLength2Code(response, HttpURLConnection.HTTP_OK);
            updatedNameElements = mapper.readValue(response[1], NameElement[].class);
            updatedNameElement = ITUtilNameElement.assertContentLengthOne(updatedNameElements);
            assertNotNull(updatedNameElement.getUuid());

            uuid = updatedNameElement.getUuid();
            nameElement.setUuid(uuid);

            ITUtilNameElement.assertContent(updatedNameElement,
                    uuid, nameElement.getParentsystemstructure(), nameElement.getParentdevicestructure(),
                    nameElement.getSystemstructure(), nameElement.getDevicestructure(),
                    nameElement.getIndex(), nameElement.getName(),
                    nameElement.getDescription(), nameElement.getStatus(), Boolean.TRUE, Boolean.FALSE,
                    "test who", nameElement.getComment());
            assertNotNull(updatedNameElement.getWhen());

            return updatedNameElement;
        } catch (IOException e) {
            fail();
        } catch (InterruptedException e) {
            fail();
        } catch (Exception e) {
            fail();
        }

        return null;
    }

    /**
     * Utility method to delete a name element and assert result.
     *
     * @param nameElement name element
     * @return deleted name element
     */
    static NameElement assertDelete(NameElement nameElement) {
        try {
            ObjectMapper mapper = new ObjectMapper();
            String[] response = null;
            UUID uuid = null;

            NameElement[] deletedNameElements = null;
            NameElement   deletedNameElement  = null;

            response = ITUtil.runShellCommand(ITUtil.curlGetPathJson(AuthorizationChoice.NONE, EndpointChoice.NAMES, "/validatedelete", "[" + mapper.writeValueAsString(nameElement) + "]"));
            ITUtil.assertResponseLength2Code(response, HttpURLConnection.HTTP_OK);
            ITUtil.assertEqualsResponseBoolean(mapper.readValue(response[1], ResponseBooleanList.class), Boolean.TRUE);

            response = ITUtil.runShellCommand(ITUtil.curlDeletePathJson(AuthorizationChoice.NONE, EndpointChoice.NAMES, "", "[" + mapper.writeValueAsString(nameElement) + "]"));
            ITUtil.assertResponseLength2Code(response, HttpURLConnection.HTTP_OK);
            deletedNameElements = mapper.readValue(response[1], NameElement[].class);
            deletedNameElement = ITUtilNameElement.assertContentLengthOne(deletedNameElements);
            assertNotNull(deletedNameElement.getUuid());

            uuid = deletedNameElement.getUuid();
            nameElement.setUuid(uuid);

            ITUtilNameElement.assertContent(deletedNameElement,
                    uuid, nameElement.getParentsystemstructure(), nameElement.getParentdevicestructure(),
                    nameElement.getSystemstructure(), nameElement.getDevicestructure(),
                    nameElement.getIndex(), nameElement.getName(),
                    nameElement.getDescription(), nameElement.getStatus(), Boolean.TRUE, Boolean.TRUE,
                    "test who", nameElement.getComment());
            assertNotNull(deletedNameElement.getWhen());

            return deletedNameElement;
        } catch (IOException e) {
            fail();
        } catch (InterruptedException e) {
            fail();
        } catch (Exception e) {
            fail();
        }

        return null;
    }

}
