/*
 * Copyright (C) 2022 European Spallation Source ERIC.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.openepics.names.docker;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.junit.jupiter.api.Assertions.fail;

import java.io.File;
import java.io.IOException;
import java.net.HttpURLConnection;
import java.util.UUID;

import org.junit.jupiter.api.Test;
import org.openepics.names.rest.beans.Status;
import org.openepics.names.rest.beans.StructureElement;
import org.openepics.names.rest.beans.Type;
import org.openepics.names.util.EnumUtil.StructureChoice;
import org.testcontainers.containers.DockerComposeContainer;
import org.testcontainers.containers.wait.strategy.Wait;
import org.testcontainers.junit.jupiter.Container;
import org.testcontainers.junit.jupiter.Testcontainers;

/**
 * Integration tests for Naming and PostgreSQL that make use of existing dockerization
 * with docker-compose.yml / Dockerfile.
 *
 * <p>
 * Focus of this class is structures endpoint and discipline.
 * </p>
 *
 * @author Lars Johansson
 */
@Testcontainers
public class StructuresDisciplineIT {

    // note
    //     StructureElement - when, who, comment - depend on requested or processed
    //         requested, requested by, requested comment
    //         processed, processed by, processed comment
    //     if less than a second between requested and processed, then considered one entry with processed
    //     history
    //         mnemonic path does not make same sense for history
    //         (very) tricky to find mnemonic path for uuid at proper time (history)
    //         therefore empty mnemonic path for history for structure
    //         one history entry if less than one second between requested and processed, otherwise two history entries
    //     attributes for entry for operations - create, update, delete, approve, cancel, reject
    //         some set client side, others set server side
    //         client side
    //             type, uuid, parent uuid, name, mnemonic, description, comment
    //         may be set client side for test purposes
    //     order of methods in test not known
    //         therefore mnemonic values chosen to not interfere with mnemonic values in other methods

    @Container
    public static final DockerComposeContainer<?> ENVIRONMENT =
        new DockerComposeContainer<>(new File("docker-compose-it-db-schema-migration.yml"))
            .waitingFor(ITUtil.NAMING, Wait.forLogMessage(".*Started NamingApplication.*", 1));

    @Test
    public void equivalenceMnemonic() {
        // purpose
        //     test mnemonic equivalence
        //
        // what
        //     read        equivalence mnemonic

        ITUtilStructureElement.assertEquivalence("Di", "D1");
    }

    @Test
    public void checkCreate() {
        // purpose
        //     test conditions for create discipline
        //         not create itself
        //
        // what - combination of
        //     read        exists in structure
        //     read        is valid to create
        //     read        validate create

        try {
            StructureElement structureElement = new StructureElement();

            ITUtilStructureElement.assertExists(Type.DISCIPLINE, "Cc", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE, "CC", Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DISCIPLINE, "Cc", Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DISCIPLINE, "CC", Boolean.TRUE);

            ITUtilStructureElement.assertValidate("[{asdf]", StructureChoice.CREATE, HttpURLConnection.HTTP_BAD_REQUEST);
            ITUtilStructureElement.assertCreate("[{asdf]", HttpURLConnection.HTTP_BAD_REQUEST);

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);
            ITUtilStructureElement.assertCreate(structureElement, HttpURLConnection.HTTP_BAD_REQUEST);

            structureElement.setType(Type.DISCIPLINE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);
            ITUtilStructureElement.assertCreate(structureElement, HttpURLConnection.HTTP_BAD_REQUEST);

            structureElement.setName("name");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);
            ITUtilStructureElement.assertCreate(structureElement, HttpURLConnection.HTTP_BAD_REQUEST);

            structureElement.setMnemonic("Cc");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);
            ITUtilStructureElement.assertCreate(structureElement, HttpURLConnection.HTTP_BAD_REQUEST);

            structureElement.setDescription("description");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);
            ITUtilStructureElement.assertCreate(structureElement, HttpURLConnection.HTTP_BAD_REQUEST);

            structureElement.setComment("comment");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.TRUE);

            structureElement.setType(null);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);
        } catch (IOException e) {
            fail();
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void checkCreateMnemonic() {
        // purpose
        //     test conditions for create discipline
        //         not create itself
        //
        // what - combination of
        //     read        validate create
        //
        // note
        //     mnemonic
        try {
            StructureElement structureElement = new StructureElement();

            structureElement.setType(Type.DISCIPLINE);
            structureElement.setName("name");
            structureElement.setDescription("description");
            structureElement.setComment("comment");

            // mnemonic rules

            structureElement.setMnemonic(null);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);

            structureElement.setMnemonic("");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);

            structureElement.setMnemonic("C");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.TRUE);

            structureElement.setMnemonic("Cc");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.TRUE);

            structureElement.setMnemonic("Ccc");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.TRUE);

            structureElement.setMnemonic("Cccc");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.TRUE);

            structureElement.setMnemonic("Ccccc");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.TRUE);

            structureElement.setMnemonic("Cccccc");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.TRUE);

            structureElement.setMnemonic("Ccccccc");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);

            structureElement.setMnemonic("Cccccccc");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);

            structureElement.setMnemonic("Ccccccccc");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);

            // mnemonic rules (2)

            structureElement.setMnemonic(" ");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);

            structureElement.setMnemonic("Dis ");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);

            structureElement.setMnemonic("Dis");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.TRUE);

            structureElement.setMnemonic("000");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.TRUE);

            structureElement.setMnemonic("Dis0");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.TRUE);

            structureElement.setMnemonic(":");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);

            structureElement.setMnemonic("Dis:");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);

            structureElement.setMnemonic("Dis:   ");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);

            structureElement.setMnemonic("1");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.TRUE);

            structureElement.setMnemonic("12");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.TRUE);

            structureElement.setMnemonic("123");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.TRUE);

            structureElement.setMnemonic("1234");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.TRUE);

            structureElement.setMnemonic("12345");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.TRUE);

            structureElement.setMnemonic("123456");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.TRUE);

            structureElement.setMnemonic("1234567");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);

            structureElement.setMnemonic("12345678");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);

            structureElement.setMnemonic("123456789");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void createApprove() {
        // purpose
        //     test create and approve discipline
        //
        // what - combination of
        //     create      create structures
        //     read        validate create
        //     read        validate approve
        //     patch       approve structures
        //
        // note
        //     create in order to approve

        try {
            StructureElement structureElement         = null;
            StructureElement createdStructureElement  = null;
            StructureElement approvedStructureElement = null;

            structureElement = new StructureElement(
                  Type.DISCIPLINE, null, null,
                  "name", "Ca", "Ca", 1,
                  "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                  null, "test who", "comment");

            ITUtilStructureElement.assertExists         (Type.DISCIPLINE, structureElement.getMnemonicpath(), Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DISCIPLINE, structureElement.getMnemonicpath(), Boolean.TRUE);

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.APPROVE, Boolean.FALSE);

            // create
            createdStructureElement = ITUtilStructureElement.assertCreate(structureElement);
            structureElement = createdStructureElement;

            ITUtilStructureElement.assertExists         (Type.DISCIPLINE, structureElement.getMnemonicpath(), Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DISCIPLINE, structureElement.getMnemonicpath(), Boolean.TRUE);

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.APPROVE, Boolean.TRUE);

            // approve
            approvedStructureElement = ITUtilStructureElement.assertApprove(structureElement);
            structureElement = approvedStructureElement;

            ITUtilStructureElement.assertExists         (Type.DISCIPLINE, structureElement.getMnemonicpath(), Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DISCIPLINE, structureElement.getMnemonicpath(), Boolean.FALSE);

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.APPROVE, Boolean.FALSE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void createCancel() {
        // purpose
        //     test create and cancel discipline
        //
        // what - combination of
        //     create      create structures
        //     read        validate create
        //     read        validate cancel
        //     patch       cancel structures
        //
        // note
        //     create in order to cancel

        try {
            StructureElement structureElement          = null;
            StructureElement createdStructureElement   = null;
            StructureElement cancelledStructureElement = null;

            structureElement = new StructureElement(
                  Type.DISCIPLINE, null, null,
                  "name", "Cc", "Cc", 1,
                  "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                  null, "test who", "comment");

            ITUtilStructureElement.assertExists         (Type.DISCIPLINE, structureElement.getMnemonicpath(), Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DISCIPLINE, structureElement.getMnemonicpath(), Boolean.TRUE);

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CANCEL, Boolean.FALSE);

            // create
            createdStructureElement = ITUtilStructureElement.assertCreate(structureElement);
            structureElement = createdStructureElement;

            ITUtilStructureElement.assertExists         (Type.DISCIPLINE, structureElement.getMnemonicpath(), Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DISCIPLINE, structureElement.getMnemonicpath(), Boolean.TRUE);

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CANCEL, Boolean.TRUE);

            // cancel
            cancelledStructureElement = ITUtilStructureElement.assertCancel(structureElement);
            structureElement = cancelledStructureElement;

            ITUtilStructureElement.assertExists         (Type.DISCIPLINE, structureElement.getMnemonicpath(), Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DISCIPLINE, structureElement.getMnemonicpath(), Boolean.TRUE);

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CANCEL, Boolean.FALSE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void createReject() {
        // purpose
        //     test create and reject discipline
        //
        // what - combination of
        //     create      create structures
        //     read        validate create
        //     read        validate reject
        //     patch       reject structures
        //
        // note
        //     create in order to reject

        try {
            StructureElement structureElement         = null;
            StructureElement createdStructureElement  = null;
            StructureElement rejectedStructureElement = null;

            structureElement = new StructureElement(
                  Type.DISCIPLINE, null, null,
                  "name", "Cr", "Cr", 1,
                  "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                  null, "test who", "comment");

            ITUtilStructureElement.assertExists         (Type.DISCIPLINE, structureElement.getMnemonicpath(), Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DISCIPLINE, structureElement.getMnemonicpath(), Boolean.TRUE);

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.REJECT, Boolean.FALSE);

            // create
            createdStructureElement = ITUtilStructureElement.assertCreate(structureElement);
            structureElement = createdStructureElement;

            ITUtilStructureElement.assertExists         (Type.DISCIPLINE, structureElement.getMnemonicpath(), Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DISCIPLINE, structureElement.getMnemonicpath(), Boolean.TRUE);

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.REJECT, Boolean.TRUE);

            // reject
            rejectedStructureElement = ITUtilStructureElement.assertReject(structureElement);
            structureElement = rejectedStructureElement;

            ITUtilStructureElement.assertExists         (Type.DISCIPLINE, structureElement.getMnemonicpath(), Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.DISCIPLINE, structureElement.getMnemonicpath(), Boolean.TRUE);

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.REJECT, Boolean.FALSE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void checkUpdate() {
        // purpose
        //     test conditions for update discipline
        //         not update
        //
        // what - combination of
        //     create      create structures
        //     read        validate create
        //     read        validate update
        //     read        validate approve
        //     patch       approve structures
        //
        // note
        //     create, approve in order to update

        try {
            UUID uuid = null;

            StructureElement structureElement         = new StructureElement();
            StructureElement approvedStructureElement = null;

            // create, approve
            structureElement = new StructureElement(
                    Type.DISCIPLINE, null, null,
                    "name", "Cu", "Cu", 1,
                    "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                    null, "test who", "test comment");
            approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
            structureElement = approvedStructureElement;
            uuid = approvedStructureElement.getUuid();

            // validate update

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.TRUE);

            structureElement.setType(null);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.FALSE);

            structureElement.setType(Type.SYSTEMGROUP);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.FALSE);

            structureElement.setType(Type.DISCIPLINE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.TRUE);

            structureElement.setUuid(null);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.FALSE);

            structureElement.setUuid(uuid);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.TRUE);

            structureElement.setName(null);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.FALSE);

            structureElement.setName("name");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.TRUE);

            structureElement.setMnemonic(null);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.FALSE);

            structureElement.setMnemonic("Cu");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.TRUE);

            structureElement.setDescription(null);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.FALSE);

            structureElement.setDescription("description");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.TRUE);

            structureElement.setComment(null);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.FALSE);

            structureElement.setComment("comment");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.TRUE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void updateApprove() {
        // purpose
        //     test update and approve discipline
        //
        // what - combination of
        //     create      create structures
        //     read        validate create
        //     read        validate update
        //     read        validate approve
        //     update      update structures
        //     patch       approve structures
        //
        // note
        //     create, approve in order to update, approve

        try {
            StructureElement structureElement         = null;
            StructureElement updatedStructureElement  = null;
            StructureElement approvedStructureElement = null;

            // create, approve
            structureElement = new StructureElement(
                  Type.DISCIPLINE, null, null,
                  "name", "Ua", "Ua", 1,
                  "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                  null, "test who", "test comment");
            approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
            structureElement = approvedStructureElement;

            structureElement.setComment("comment update approve check");

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.APPROVE, Boolean.FALSE);

            // update
            updatedStructureElement = ITUtilStructureElement.assertUpdate(structureElement);
            structureElement = updatedStructureElement;

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.APPROVE, Boolean.TRUE);

            // approve
            approvedStructureElement = ITUtilStructureElement.assertApprove(structureElement);

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.APPROVE, Boolean.FALSE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void updateCancel() {
        // purpose
        //     test update and cancel discipline
        //
        // what - combination of
        //     create      create structures
        //     read        validate create
        //     read        validate update
        //     read        validate approve
        //     read        validate cancel
        //     update      update structures
        //     patch       approve structures
        //     patch       cancel structures
        //
        // note
        //     create, approve in order to update, cancel

        try {
            StructureElement structureElement          = null;
            StructureElement updatedStructureElement   = null;
            StructureElement approvedStructureElement  = null;
            StructureElement cancelledStructureElement = null;

            // create, approve
            structureElement = new StructureElement(
                    Type.DISCIPLINE, null, null,
                    "name", "Uc", "Uc", 1,
                    "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                    null, "test who", "test comment");
            approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
            structureElement = approvedStructureElement;

            structureElement.setComment("comment update cancel check");

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CANCEL, Boolean.FALSE);

            // update
            updatedStructureElement = ITUtilStructureElement.assertUpdate(structureElement);
            structureElement = updatedStructureElement;

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CANCEL, Boolean.TRUE);

            // cancel
            cancelledStructureElement = ITUtilStructureElement.assertCancel(structureElement);
            structureElement = cancelledStructureElement;

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CANCEL, Boolean.FALSE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void updateReject() {
        // purpose
        //     test update and reject discipline
        //
        // what - combination of
        //     create      create structures
        //     read        validate create
        //     read        validate update
        //     read        validate approve
        //     read        validate reject
        //     update      update structures
        //     patch       approve structures
        //     patch       reject structures
        //
        // note
        //     create, approve in order to update, reject

        try {
            StructureElement structureElement         = null;
            StructureElement updatedStructureElement  = null;
            StructureElement approvedStructureElement = null;
            StructureElement rejectedStructureElement = null;

            // create, approve
            structureElement = new StructureElement(
                    Type.DISCIPLINE, null, null,
                    "name", "Ur", "Ur", 1,
                    "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                    null, "test who", "test comment");
            approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
            structureElement = approvedStructureElement;

            structureElement.setComment("comment update reject check");

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.REJECT, Boolean.FALSE);

            // update
            updatedStructureElement = ITUtilStructureElement.assertUpdate(structureElement);
            structureElement = updatedStructureElement;

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.REJECT, Boolean.TRUE);

            // reject
            rejectedStructureElement = ITUtilStructureElement.assertReject(structureElement);
            structureElement = rejectedStructureElement;

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.REJECT, Boolean.FALSE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void checkDelete() {
        // purpose
        //     test conditions for delete discipline
        //         not delete
        //
        // what - combination of
        //     create      create structures
        //     read        validate create
        //     read        validate delete
        //     read        validate approve
        //     patch       approve structures
        //
        // note
        //     create, approve in order to delete

        try {
            UUID uuid = null;

            StructureElement   structureElement         = new StructureElement();
            StructureElement   approvedStructureElement = null;

            // create, approve
            structureElement = new StructureElement(
                    Type.DISCIPLINE, null, null,
                    "name", "Cd", "Cd", 1,
                    "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                    null, "test who", "test comment");
            approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
            structureElement = approvedStructureElement;
            uuid = approvedStructureElement.getUuid();

            // validate delete

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.TRUE);

            structureElement.setType(null);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.FALSE);

            structureElement.setType(Type.SYSTEMGROUP);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.FALSE);

            structureElement.setType(Type.DISCIPLINE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.TRUE);

            structureElement.setUuid(null);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.FALSE);

            structureElement.setUuid(uuid);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.TRUE);

            structureElement.setName(null);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.FALSE);

            structureElement.setName("name");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.TRUE);

            structureElement.setMnemonic(null);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.FALSE);

            structureElement.setMnemonic("Cd");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.TRUE);

            structureElement.setDescription(null);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.FALSE);

            structureElement.setDescription("description");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.TRUE);

            structureElement.setComment(null);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.FALSE);

            structureElement.setComment("comment");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.TRUE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void deleteApprove() {
        // purpose
        //     test delete and approve discipline
        //
        // what - combination of
        //     create      create structures
        //     read        validate create
        //     read        validate delete
        //     read        validate approve
        //     delete      delete structures
        //     patch       approve structures
        //
        // note
        //     create, approve in order to delete, approve

        try {
            StructureElement structureElement         = null;
            StructureElement deletedStructureElement  = null;
            StructureElement approvedStructureElement = null;

            // create, approve
            structureElement = new StructureElement(
                  Type.DISCIPLINE, null, null,
                  "name", "Da", "Da", 1,
                  "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                  null, "test who", "test comment");
            approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
            structureElement = approvedStructureElement;

            structureElement.setComment("comment delete approve check");

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.APPROVE, Boolean.FALSE);

            // delete
            structureElement.setDeleted(Boolean.TRUE);
            deletedStructureElement = ITUtilStructureElement.assertDelete(structureElement);
            structureElement = deletedStructureElement;

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.APPROVE, Boolean.TRUE);

            // approve
            approvedStructureElement = ITUtilStructureElement.assertApprove(structureElement);

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.APPROVE, Boolean.FALSE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void deleteCancel() {
        // purpose
        //     test delete and cancel discipline
        //
        // what - combination of
        //     create      create structures
        //     read        validate create
        //     read        validate delete
        //     read        validate approve
        //     read        validate cancel
        //     update      delete structures
        //     patch       approve structures
        //     patch       cancel structures
        //
        // note
        //     create, approve in order to delete, cancel

        try {
            StructureElement structureElement          = null;
            StructureElement deletedStructureElement   = null;
            StructureElement approvedStructureElement  = null;
            StructureElement cancelledStructureElement = null;

            // create, approve
            structureElement = new StructureElement(
                    Type.DISCIPLINE, null, null,
                    "name", "Dc", "Dc", 1,
                    "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                    null, "test who", "test comment");
            approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
            structureElement = approvedStructureElement;

            structureElement.setComment("comment update delete check");

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CANCEL, Boolean.FALSE);

            // delete
            deletedStructureElement = ITUtilStructureElement.assertDelete(structureElement);
            structureElement = deletedStructureElement;

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CANCEL, Boolean.TRUE);

            // cancel
            cancelledStructureElement = ITUtilStructureElement.assertCancel(structureElement);
            structureElement = cancelledStructureElement;

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CANCEL, Boolean.FALSE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void deleteReject() {
        // purpose
        //     test delete and reject discipline
        //
        // what - combination of
        //     create      create structures
        //     read        validate create
        //     read        validate delete
        //     read        validate approve
        //     read        validate reject
        //     update      delete structures
        //     patch       approve structures
        //     patch       reject structures
        //
        // note
        //     create, approve in order to delete, reject

        try {
            StructureElement structureElement         = null;
            StructureElement deletedStructureElement  = null;
            StructureElement approvedStructureElement = null;
            StructureElement rejectedStructureElement = null;

            // create, approve
            structureElement = new StructureElement(
                    Type.DISCIPLINE, null, null,
                    "name", "Dr", "Dr", 1,
                    "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                    null, "test who", "test comment");
            approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
            structureElement = approvedStructureElement;

            structureElement.setComment("comment delete reject check");

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.REJECT, Boolean.FALSE);

            // delete
            deletedStructureElement = ITUtilStructureElement.assertDelete(structureElement);
            structureElement = deletedStructureElement;

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.REJECT, Boolean.TRUE);

            // reject
            rejectedStructureElement = ITUtilStructureElement.assertReject(structureElement);
            structureElement = rejectedStructureElement;

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.REJECT, Boolean.FALSE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void readSearchHistoryApprove() {
        // purpose
        //     test read discipline in various ways for create, approve
        //         search
        //         latest
        //         history
        //
        // note
        //     create (and more) to read (with content)

        try {
            StructureElement   structureElement         = null;
            StructureElement   createdStructureElement  = null;
            StructureElement   approvedStructureElement = null;
            int length = 0;

            structureElement = new StructureElement(
                  Type.DISCIPLINE, null, null,
                  "name", "Rsha", "Rsha", 1,
                  "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                  null, "test who", "comment");

            // create
            createdStructureElement = ITUtilStructureElement.assertCreate(structureElement);
            structureElement = createdStructureElement;

            // read (1)
            length = ITUtilStructureElement.assertRead("/DISCIPLINE", -1, 1);

            ITUtilStructureElement.assertRead("/DISCIPLINE?statuses=PENDING",   -1, 1);
            ITUtilStructureElement.assertRead("/DISCIPLINE?statuses=APPROVED",  -1, 0);
            ITUtilStructureElement.assertRead("/DISCIPLINE?statuses=CANCELLED", -1, 0);
            ITUtilStructureElement.assertRead("/DISCIPLINE?statuses=REJECTED",  -1, 0);
            ITUtilStructureElement.assertRead("/DISCIPLINE?statuses=PENDING&queryFields=MNEMONIC&queryValues=Rsha",   1, 1);
            ITUtilStructureElement.assertRead("/DISCIPLINE?statuses=APPROVED&queryFields=MNEMONIC&queryValues=Rsha",  0, 0);
            ITUtilStructureElement.assertRead("/DISCIPLINE?statuses=CANCELLED&queryFields=MNEMONIC&queryValues=Rsha", 0, 0);
            ITUtilStructureElement.assertRead("/DISCIPLINE?statuses=REJECTED&queryFields=MNEMONIC&queryValues=Rsha",  0, 0);
            ITUtilStructureElement.assertRead("/DISCIPLINE?queryFields=UUID&queryValues=" + createdStructureElement.getUuid().toString(), 1, 1);

            ITUtilStructureElement.assertRead("/mnemonic/Rsha",     0, 0);

            ITUtilStructureElement.assertRead("/mnemonicpath/Rsha", 0, 0);

            ITUtilStructureElement.assertRead("/history/" + createdStructureElement.getUuid().toString(), -1, 1);

            ITUtilStructureElement.assertExists(Type.DISCIPLINE, "Rsha", Boolean.FALSE);

            // approve
            approvedStructureElement = ITUtilStructureElement.assertApprove(structureElement);

            // read (2)
            assertEquals(length, ITUtilStructureElement.assertRead("/DISCIPLINE", -1, 1));

            ITUtilStructureElement.assertRead("/DISCIPLINE?statuses=PENDING",   -1, 0);
            ITUtilStructureElement.assertRead("/DISCIPLINE?statuses=APPROVED",  -1, 1);
            ITUtilStructureElement.assertRead("/DISCIPLINE?statuses=CANCELLED", -1, 0);
            ITUtilStructureElement.assertRead("/DISCIPLINE?statuses=REJECTED",  -1, 0);
            ITUtilStructureElement.assertRead("/DISCIPLINE?statuses=PENDING&queryFields=MNEMONIC&queryValues=Rsha",   0, 0);
            ITUtilStructureElement.assertRead("/DISCIPLINE?statuses=APPROVED&queryFields=MNEMONIC&queryValues=Rsha",  1, 1);
            ITUtilStructureElement.assertRead("/DISCIPLINE?statuses=CANCELLED&queryFields=MNEMONIC&queryValues=Rsha", 0, 0);
            ITUtilStructureElement.assertRead("/DISCIPLINE?statuses=REJECTED&queryFields=MNEMONIC&queryValues=Rsha",  0, 0);
            ITUtilStructureElement.assertRead("/DISCIPLINE?queryFields=UUID&queryValues=" + approvedStructureElement.getUuid().toString(), 1, 1);

            ITUtilStructureElement.assertRead("/mnemonic/Rsha",     1, 1);

            ITUtilStructureElement.assertRead("/mnemonicpath/Rsha", 1, 1);

            ITUtilStructureElement.assertRead("/history/" + approvedStructureElement.getUuid().toString(), -1, 1);

            ITUtilStructureElement.assertExists(Type.DISCIPLINE, "Rsha", Boolean.TRUE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void readSearchHistoryCancel() {
        // purpose
        //     test read discipline in various ways for create, cancel
        //         search
        //         latest
        //         history
        //
        // note
        //     create (and more) to read (with content)

        try {
            StructureElement   structureElement          = null;
            StructureElement   createdStructureElement   = null;
            StructureElement   cancelledStructureElement = null;
            int length = 0;

            structureElement = new StructureElement(
                  Type.DISCIPLINE, null, null,
                  "name", "Rshc", "Rshc", 1,
                  "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                  null, "test who", "comment");

            // create
            createdStructureElement = ITUtilStructureElement.assertCreate(structureElement);
            structureElement = createdStructureElement;

            // read (1)
            length = ITUtilStructureElement.assertRead("/DISCIPLINE", -1, 1);

            ITUtilStructureElement.assertRead("/DISCIPLINE?statuses=PENDING",   -1, 1);
            ITUtilStructureElement.assertRead("/DISCIPLINE?statuses=APPROVED",  -1, 0);
            ITUtilStructureElement.assertRead("/DISCIPLINE?statuses=CANCELLED", -1, 0);
            ITUtilStructureElement.assertRead("/DISCIPLINE?statuses=REJECTED",  -1, 0);
            ITUtilStructureElement.assertRead("/DISCIPLINE?statuses=PENDING&queryFields=MNEMONIC&queryValues=Rshc",   1, 1);
            ITUtilStructureElement.assertRead("/DISCIPLINE?statuses=APPROVED&queryFields=MNEMONIC&queryValues=Rshc",  0, 0);
            ITUtilStructureElement.assertRead("/DISCIPLINE?statuses=CANCELLED&queryFields=MNEMONIC&queryValues=Rshc", 0, 0);
            ITUtilStructureElement.assertRead("/DISCIPLINE?statuses=REJECTED&queryFields=MNEMONIC&queryValues=Rshc",  0, 0);
            ITUtilStructureElement.assertRead("/DISCIPLINE?queryFields=UUID&queryValues=" + createdStructureElement.getUuid().toString(), 1, 1);

            ITUtilStructureElement.assertRead("/mnemonic/Rshc",     0, 0);

            ITUtilStructureElement.assertRead("/mnemonicpath/Rshc", 0, 0);

            ITUtilStructureElement.assertRead("/history/" + createdStructureElement.getUuid().toString(), -1, 1);

            ITUtilStructureElement.assertExists(Type.DISCIPLINE, "Rshc", Boolean.FALSE);

            // cancel
            cancelledStructureElement = ITUtilStructureElement.assertCancel(structureElement);

            // read (2)
            assertEquals(length, ITUtilStructureElement.assertRead("/DISCIPLINE", -1, 1));

            ITUtilStructureElement.assertRead("/DISCIPLINE?statuses=PENDING",   -1, 0);
            ITUtilStructureElement.assertRead("/DISCIPLINE?statuses=APPROVED",  -1, 0);
            ITUtilStructureElement.assertRead("/DISCIPLINE?statuses=CANCELLED", -1, 1);
            ITUtilStructureElement.assertRead("/DISCIPLINE?statuses=REJECTED",  -1, 0);
            ITUtilStructureElement.assertRead("/DISCIPLINE?statuses=PENDING&queryFields=MNEMONIC&queryValues=Rshc",   0, 0);
            ITUtilStructureElement.assertRead("/DISCIPLINE?statuses=APPROVED&queryFields=MNEMONIC&queryValues=Rshc",  0, 0);
            ITUtilStructureElement.assertRead("/DISCIPLINE?statuses=CANCELLED&queryFields=MNEMONIC&queryValues=Rshc", 1, 1);
            ITUtilStructureElement.assertRead("/DISCIPLINE?statuses=REJECTED&queryFields=MNEMONIC&queryValues=Rshc",  0, 0);
            ITUtilStructureElement.assertRead("/DISCIPLINE?queryFields=UUID&queryValues=" + cancelledStructureElement.getUuid().toString(), 1, 1);

            ITUtilStructureElement.assertRead("/mnemonic/Rshc",     0, 0);

            ITUtilStructureElement.assertRead("/mnemonicpath/Rshc", 0, 0);

            ITUtilStructureElement.assertRead("/history/" + cancelledStructureElement.getUuid().toString(), -1, 1);

            ITUtilStructureElement.assertExists(Type.DISCIPLINE, "Rshc", Boolean.FALSE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void readSearchHistoryReject() {
        // purpose
        //     test read discipline in various ways for create, reject
        //         search
        //         latest
        //         history
        //
        // note
        //     create (and more) to read (with content)

        try {
            StructureElement   structureElement         = null;
            StructureElement   createdStructureElement  = null;
            StructureElement   rejectedStructureElement = null;
            int length = 0;

            structureElement = new StructureElement(
                  Type.DISCIPLINE, null, null,
                  "name", "Rshr", "Rshr", 1,
                  "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                  null, "test who", "comment");

            // create
            createdStructureElement = ITUtilStructureElement.assertCreate(structureElement);
            structureElement = createdStructureElement;

            // read (1)
            length = ITUtilStructureElement.assertRead("/DISCIPLINE", -1, 1);

            ITUtilStructureElement.assertRead("/DISCIPLINE?statuses=PENDING",   -1, 1);
            ITUtilStructureElement.assertRead("/DISCIPLINE?statuses=APPROVED",  -1, 0);
            ITUtilStructureElement.assertRead("/DISCIPLINE?statuses=CANCELLED", -1, 0);
            ITUtilStructureElement.assertRead("/DISCIPLINE?statuses=REJECTED",  -1, 0);
            ITUtilStructureElement.assertRead("/DISCIPLINE?statuses=PENDING&queryFields=MNEMONIC&queryValues=Rshr",   1, 1);
            ITUtilStructureElement.assertRead("/DISCIPLINE?statuses=APPROVED&queryFields=MNEMONIC&queryValues=Rshr",  0, 0);
            ITUtilStructureElement.assertRead("/DISCIPLINE?statuses=CANCELLED&queryFields=MNEMONIC&queryValues=Rshr", 0, 0);
            ITUtilStructureElement.assertRead("/DISCIPLINE?statuses=REJECTED&queryFields=MNEMONIC&queryValues=Rshr",  0, 0);
            ITUtilStructureElement.assertRead("/DISCIPLINE?queryFields=UUID&queryValues=" + createdStructureElement.getUuid().toString(), 1, 1);

            ITUtilStructureElement.assertRead("/mnemonic/Rshr",     0, 0);

            ITUtilStructureElement.assertRead("/mnemonicpath/Rshr", 0, 0);

            ITUtilStructureElement.assertRead("/history/" + createdStructureElement.getUuid().toString(), -1, 1);

            ITUtilStructureElement.assertExists(Type.DISCIPLINE, "Rshr", Boolean.FALSE);

            // reject
            rejectedStructureElement = ITUtilStructureElement.assertReject(structureElement);

            // read (2)
            assertEquals(length, ITUtilStructureElement.assertRead("/DISCIPLINE", -1, 1));

            ITUtilStructureElement.assertRead("/DISCIPLINE?statuses=PENDING",   -1, 0);
            ITUtilStructureElement.assertRead("/DISCIPLINE?statuses=APPROVED",  -1, 0);
            ITUtilStructureElement.assertRead("/DISCIPLINE?statuses=CANCELLED", -1, 0);
            ITUtilStructureElement.assertRead("/DISCIPLINE?statuses=REJECTED",  -1, 1);
            ITUtilStructureElement.assertRead("/DISCIPLINE?statuses=PENDING&queryFields=MNEMONIC&queryValues=Rshr",   0, 0);
            ITUtilStructureElement.assertRead("/DISCIPLINE?statuses=APPROVED&queryFields=MNEMONIC&queryValues=Rshr",  0, 0);
            ITUtilStructureElement.assertRead("/DISCIPLINE?statuses=CANCELLED&queryFields=MNEMONIC&queryValues=Rshr", 0, 0);
            ITUtilStructureElement.assertRead("/DISCIPLINE?statuses=REJECTED&queryFields=MNEMONIC&queryValues=Rshr",  1, 1);
            ITUtilStructureElement.assertRead("/DISCIPLINE?queryFields=UUID&queryValues=" + rejectedStructureElement.getUuid().toString(), 1, 1);

            ITUtilStructureElement.assertRead("/mnemonic/Rshr",     0, 0);

            ITUtilStructureElement.assertRead("/mnemonicpath/Rshr", 0, 0);

            ITUtilStructureElement.assertRead("/history/" + rejectedStructureElement.getUuid().toString(), -1, 1);

            ITUtilStructureElement.assertExists(Type.DISCIPLINE, "Rshr", Boolean.FALSE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void readSearchHistoryStatusLatest1() {
        // purpose
        //     test read discipline in various ways
        //         status
        //         latest
        //       ( deleted )
        //         history
        //
        // what
        //     entries with different statuses
        //
        // note
        //     create (and more) to read (with content)
        //     4 different lines  of uuid with combinations of status and latest
        //         1 entry for each line of uuid
        //         in particular different statuses for last entry in lines of uuid
        //         mnemonic - P, C, R, A (pending, cancelled, rejected, approved)
        //
        // note!
        //     not only
        //         exclude APPROVED and not latest
        //     not only
        //         exclude earlier than APPROVED and latest (considered history)
        //         include PENDING and not latest           (considered future)
        //     instead
        //         exclude earlier than APPROVED and latest (considered history)
        //       ( include if latest )
        //         include if latest not exists             (considered present + future)

        StructureElement structureElement         = null;
        StructureElement responseStructureElement = null;
        UUID uuid, uuid2, uuid3, uuid4 = null;

        // a number of entries for which the last entry has status PENDING
        structureElement = new StructureElement(
                Type.DISCIPLINE, null, null,
                "name", "P1", "P1", 1,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment 1");
        responseStructureElement = ITUtilStructureElement.assertCreate(structureElement);
        uuid = responseStructureElement.getUuid();

        // a number of entries for which the last entry has status CANCELLED
        structureElement = new StructureElement(
                Type.DISCIPLINE, null, null,
                "name", "C1", "C1", 1,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment 1");
        responseStructureElement = ITUtilStructureElement.assertCreateCancel(structureElement);
        uuid2 = responseStructureElement.getUuid();

        // a number of entries for which the last entry has status REJECTED
        structureElement = new StructureElement(
                Type.DISCIPLINE, null, null,
                "name", "R1", "R1", 1,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment 1");
        responseStructureElement = ITUtilStructureElement.assertCreateReject(structureElement);
        uuid3 = responseStructureElement.getUuid();

        // a number of entries for which the last entry has status APPROVED
        structureElement = new StructureElement(
                Type.DISCIPLINE, null, null,
                "name", "A1", "A1", 1,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment 1");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        uuid4 = responseStructureElement.getUuid();

        try {
            // from first structure element
            assertNotNull(uuid);

            // note
            //     exclude earlier than APPROVED and latest (considered history)
            //   ( include if latest )
            //     include if latest not exists             (considered present + future)

            ITUtilStructureElement.assertRead("/DISCIPLINE?queryFields=UUID&queryValues=" + uuid.toString(),  1, 1);
            ITUtilStructureElement.assertRead("/DISCIPLINE?queryFields=UUID&queryValues=" + uuid2.toString(), 1, 1);
            ITUtilStructureElement.assertRead("/DISCIPLINE?queryFields=UUID&queryValues=" + uuid3.toString(), 1, 1);
            ITUtilStructureElement.assertRead("/DISCIPLINE?queryFields=UUID&queryValues=" + uuid4.toString(), 1, 1);

            ITUtilStructureElement.assertRead("/mnemonic/P1", 0, 0);
            ITUtilStructureElement.assertRead("/mnemonic/C1", 0, 0);
            ITUtilStructureElement.assertRead("/mnemonic/R1", 0, 0);
            ITUtilStructureElement.assertRead("/mnemonic/A1", 1, 1);

            // normally expected number of entries are 1, 2, 2, 2
            // since within 1 second from requested to processed, expected number of entries 1, 1, 1, 1
            ITUtilStructureElement.assertRead("/history/" + uuid.toString(),  1, 1);
            ITUtilStructureElement.assertRead("/history/" + uuid2.toString(), 1, 1);
            ITUtilStructureElement.assertRead("/history/" + uuid3.toString(), 1, 1);
            ITUtilStructureElement.assertRead("/history/" + uuid4.toString(), 1, 1);

            ITUtilStructureElement.assertExists(Type.DISCIPLINE, "P1", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE, "C1", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE, "R1", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE, "A1", Boolean.TRUE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void readSearchHistoryStatusLatest9() {
        // purpose
        //     test read discipline in various ways
        //         status
        //         latest
        //       ( deleted )
        //         history
        //
        // what
        //     entries with different statuses
        //
        // note
        //     create (and more) to read (with content)
        //     4 different lines  of uuid with combinations of status and latest
        //         9 entries for each line of uuid
        //         in particular different statuses for last entry in lines of uuid
        //         mnemonic - P, C, R, A (pending, cancelled, rejected, approved)
        //
        // note!
        //     not only
        //         exclude APPROVED and not latest
        //     not only
        //         exclude earlier than APPROVED and latest (considered history)
        //         include PENDING and not latest           (considered future)
        //     instead
        //         exclude earlier than APPROVED and latest (considered history)
        //       ( include if latest )
        //         include if latest not exists             (considered present + future)

        StructureElement structureElement         = null;
        StructureElement responseStructureElement = null;
        UUID uuid, uuid2, uuid3, uuid4 = null;

        // a number of entries for which the last entry has status PENDING
        structureElement = new StructureElement(
                Type.DISCIPLINE, null, null,
                "name", "P9", "P9", 1,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment 1");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        uuid = responseStructureElement.getUuid();

        responseStructureElement.setComment("test comment 2");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("test comment 3");
        responseStructureElement = ITUtilStructureElement.assertUpdateCancel(responseStructureElement);
        responseStructureElement.setComment("test comment 4");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("test comment 5");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("test comment 6");
        responseStructureElement = ITUtilStructureElement.assertUpdateReject(responseStructureElement);
        responseStructureElement.setComment("test comment 7");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("test comment 8");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("test comment 9");
        responseStructureElement = ITUtilStructureElement.assertUpdate(responseStructureElement);

        // a number of entries for which the last entry has status CANCELLED
        structureElement = new StructureElement(
                Type.DISCIPLINE, null, null,
                "name", "C9", "C9", 1,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment 1");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        uuid2 = responseStructureElement.getUuid();

        responseStructureElement.setComment("test comment 2");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("test comment 3");
        responseStructureElement = ITUtilStructureElement.assertUpdateCancel(responseStructureElement);
        responseStructureElement.setComment("test comment 4");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("test comment 5");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("test comment 6");
        responseStructureElement = ITUtilStructureElement.assertUpdateReject(responseStructureElement);
        responseStructureElement.setComment("test comment 7");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("test comment 8");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("test comment 9");
        responseStructureElement = ITUtilStructureElement.assertUpdateCancel(responseStructureElement);

        // a number of entries for which the last entry has status REJECTED
        structureElement = new StructureElement(
                Type.DISCIPLINE, null, null,
                "name", "R9", "R9", 1,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment 1");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        uuid3 = responseStructureElement.getUuid();

        responseStructureElement.setComment("test comment 2");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("test comment 3");
        responseStructureElement = ITUtilStructureElement.assertUpdateCancel(responseStructureElement);
        responseStructureElement.setComment("test comment 4");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("test comment 5");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("test comment 6");
        responseStructureElement = ITUtilStructureElement.assertUpdateReject(responseStructureElement);
        responseStructureElement.setComment("test comment 7");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("test comment 8");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("test comment 9");
        responseStructureElement = ITUtilStructureElement.assertUpdateReject(responseStructureElement);

        // a number of entries for which the last entry has status APPROVED
        structureElement = new StructureElement(
                Type.DISCIPLINE, null, null,
                "name", "A9", "A9", 1,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment 1");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        uuid4 = responseStructureElement.getUuid();

        responseStructureElement.setComment("test comment 2");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("test comment 3");
        responseStructureElement = ITUtilStructureElement.assertUpdateCancel(responseStructureElement);
        responseStructureElement.setComment("test comment 4");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("test comment 5");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("test comment 6");
        responseStructureElement = ITUtilStructureElement.assertUpdateReject(responseStructureElement);
        responseStructureElement.setComment("test comment 7");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("test comment 8");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("test comment 9");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);

        try {
            // from first structure element
            assertNotNull(uuid);

            // note
            //     exclude earlier than APPROVED and latest (considered history)
            //   ( include if latest )
            //     include if latest not exists             (considered present + future)

            ITUtilStructureElement.assertRead("/DISCIPLINE?queryFields=UUID&queryValues=" + uuid.toString(),  2, 2);
            ITUtilStructureElement.assertRead("/DISCIPLINE?queryFields=UUID&queryValues=" + uuid2.toString(), 2, 2);
            ITUtilStructureElement.assertRead("/DISCIPLINE?queryFields=UUID&queryValues=" + uuid3.toString(), 2, 2);
            ITUtilStructureElement.assertRead("/DISCIPLINE?queryFields=UUID&queryValues=" + uuid4.toString(), 1, 1);

            ITUtilStructureElement.assertRead("/mnemonic/P9", 1, 1);
            ITUtilStructureElement.assertRead("/mnemonic/C9", 1, 1);
            ITUtilStructureElement.assertRead("/mnemonic/R9", 1, 1);
            ITUtilStructureElement.assertRead("/mnemonic/A9", 1, 1);

            // normally expected number of entries are 17, 18, 18, 18
            // since within 1 second from requested to processed, expected number of entries 9, 9, 9, 9
            ITUtilStructureElement.assertRead("/history/" + uuid.toString(),  9, 9);
            ITUtilStructureElement.assertRead("/history/" + uuid2.toString(), 9, 9);
            ITUtilStructureElement.assertRead("/history/" + uuid3.toString(), 9, 9);
            ITUtilStructureElement.assertRead("/history/" + uuid4.toString(), 9, 9);

            ITUtilStructureElement.assertExists(Type.DISCIPLINE, "P9", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE, "C9", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE, "R9", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE, "A9", Boolean.TRUE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void readSearchStatusDeletedChildren() {
        // purpose
        //     test read discipline in various ways
        //         status
        //       ( latest )
        //         deleted
        //         children
        //
        // what
        //     entries with different statuses
        //
        // note
        //     create (and more) to read (with content)
        //     querying for Status.APPROVED means latest approved

        StructureElement structureElement         = null;
        StructureElement responseStructureElement = null;
        UUID uuid = null;

        structureElement = new StructureElement(
                Type.DISCIPLINE, null, null,
                "name", "AA1", "AA1", 1,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        uuid = responseStructureElement.getUuid();

        structureElement = new StructureElement(
                Type.DISCIPLINE, null, null,
                "name", "AA2", "AA2", 1,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);

        structureElement = new StructureElement(
                Type.DISCIPLINE, null, null,
                "name", "AA3", "AA3", 1,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);

        structureElement = new StructureElement(
                Type.DISCIPLINE, null, null,
                "name", "AA4", "AA4", 1,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);

        structureElement = new StructureElement(
                Type.DISCIPLINE, null, null,
                "name", "AA5", "AA5", 1,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);

        structureElement = new StructureElement(
                Type.DISCIPLINE, null, null,
                "name", "AB1", "AB1", 1,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateCancel(structureElement);

        structureElement = new StructureElement(
                Type.DISCIPLINE, null, null,
                "name", "AB2", "AB2", 1,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateCancel(structureElement);

        structureElement = new StructureElement(
                Type.DISCIPLINE, null, null,
                "name", "AB3", "AB3", 1,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateCancel(structureElement);

        structureElement = new StructureElement(
                Type.DISCIPLINE, null, null,
                "name", "AB4", "AB4", 1,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateCancel(structureElement);

        structureElement = new StructureElement(
                Type.DISCIPLINE, null, null,
                "name", "AB5", "AB5", 1,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateCancel(structureElement);

        structureElement = new StructureElement(
                Type.DISCIPLINE, null, null,
                "name", "AC1", "AC1", 1,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateReject(structureElement);

        structureElement = new StructureElement(
                Type.DISCIPLINE, null, null,
                "name", "AC2", "AC2", 1,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateReject(structureElement);

        structureElement = new StructureElement(
                Type.DISCIPLINE, null, null,
                "name", "AC3", "AC3", 1,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateReject(structureElement);

        structureElement = new StructureElement(
                Type.DISCIPLINE, null, null,
                "name", "AC4", "AC4", 1,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateReject(structureElement);

        structureElement = new StructureElement(
                Type.DISCIPLINE, null, null,
                "name", "AC5", "AC5", 1,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateReject(structureElement);

        structureElement = new StructureElement(
                Type.DISCIPLINE, null, null,
                "name", "AD1", "AD1", 1,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreate(structureElement);

        structureElement = new StructureElement(
                Type.DISCIPLINE, null, null,
                "name", "AD2", "AD2", 1,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreate(structureElement);

        structureElement = new StructureElement(
                Type.DISCIPLINE, null, null,
                "name", "AD3", "AD3", 1,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreate(structureElement);

        structureElement = new StructureElement(
                Type.DISCIPLINE, null, null,
                "name", "AD4", "AD4", 1,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreate(structureElement);

        structureElement = new StructureElement(
                Type.DISCIPLINE, null, null,
                "name", "AD5", "AD5", 1,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreate(structureElement);

        String description2 = "some other description";
        String comment2 = "some other comment";
        String description3 = "more description";
        String comment3 = "more comment";

        structureElement = new StructureElement(
                Type.DISCIPLINE, null, null,
                "name", "AE1", "AE1", 1,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDescription(description2);
        structureElement.setComment(comment2);
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDescription(description3);
        structureElement.setComment(comment3);
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDeleted(Boolean.FALSE);
        responseStructureElement = ITUtilStructureElement.assertDeleteReject(structureElement);

        structureElement = new StructureElement(
                Type.DISCIPLINE, null, null,
                "name", "AE2", "AE2", 1,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDescription(description2);
        structureElement.setComment(comment2);
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDescription(description3);
        structureElement.setComment(comment3);
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDeleted(Boolean.FALSE);
        responseStructureElement = ITUtilStructureElement.assertDeleteReject(structureElement);

        structureElement = new StructureElement(
                Type.DISCIPLINE, null, null,
                "name", "AE3", "AE3", 1,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDescription(description2);
        structureElement.setComment(comment2);
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDescription(description3);
        structureElement.setComment(comment3);
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDeleted(Boolean.FALSE);
        responseStructureElement = ITUtilStructureElement.assertDeleteReject(structureElement);

        structureElement = new StructureElement(
                Type.DISCIPLINE, null, null,
                "name", "AE4", "AE4", 1,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDescription(description2);
        structureElement.setComment(comment2);
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDescription(description3);
        structureElement.setComment(comment3);
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDeleted(Boolean.FALSE);
        responseStructureElement = ITUtilStructureElement.assertDeleteReject(structureElement);

        structureElement = new StructureElement(
                Type.DISCIPLINE, null, null,
                "name", "AE5", "AE5", 1,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDescription(description2);
        structureElement.setComment(comment2);
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDescription(description3);
        structureElement.setComment(comment3);
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDeleted(Boolean.FALSE);
        responseStructureElement = ITUtilStructureElement.assertDeleteReject(structureElement);

        structureElement = new StructureElement(
                Type.DISCIPLINE, null, null,
                "name", "AF1", "AF1", 1,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDeleted(Boolean.FALSE);
        responseStructureElement = ITUtilStructureElement.assertDeleteApprove(structureElement);

        structureElement = new StructureElement(
                Type.DISCIPLINE, null, null,
                "name", "AF2", "AF2", 1,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDeleted(Boolean.FALSE);
        responseStructureElement = ITUtilStructureElement.assertDeleteApprove(structureElement);

        structureElement = new StructureElement(
                Type.DISCIPLINE, null, null,
                "name", "AF3", "AF3", 1,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDeleted(Boolean.FALSE);
        responseStructureElement = ITUtilStructureElement.assertDeleteApprove(structureElement);

        structureElement = new StructureElement(
                Type.DISCIPLINE, null, null,
                "name", "AF4", "AF4", 1,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDeleted(Boolean.FALSE);
        responseStructureElement = ITUtilStructureElement.assertDeleteApprove(structureElement);

        structureElement = new StructureElement(
                Type.DISCIPLINE, null, null,
                "name", "AF5", "AF5", 1,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDeleted(Boolean.FALSE);
        responseStructureElement = ITUtilStructureElement.assertDeleteApprove(structureElement);

        structureElement = new StructureElement(
                Type.DISCIPLINE, null, null,
                "name", "AG1", "AG1", 1,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDeleted(Boolean.FALSE);
        responseStructureElement = ITUtilStructureElement.assertDelete(structureElement);

        structureElement = new StructureElement(
                Type.DISCIPLINE, null, null,
                "name", "AG2", "AG2", 1,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDeleted(Boolean.FALSE);
        responseStructureElement = ITUtilStructureElement.assertDelete(structureElement);

        structureElement = new StructureElement(
                Type.DISCIPLINE, null, null,
                "name", "AG3", "AG3", 1,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDeleted(Boolean.FALSE);
        responseStructureElement = ITUtilStructureElement.assertDelete(structureElement);

        structureElement = new StructureElement(
                Type.DISCIPLINE, null, null,
                "name", "AG4", "AG4", 1,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDeleted(Boolean.FALSE);
        responseStructureElement = ITUtilStructureElement.assertDelete(structureElement);

        structureElement = new StructureElement(
                Type.DISCIPLINE, null, null,
                "name", "AG5", "AG5", 1,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDeleted(Boolean.FALSE);
        responseStructureElement = ITUtilStructureElement.assertDelete(structureElement);

        // 60 discipline entries

        try {
            // from first structure element
            assertNotNull(uuid);

            ITUtilStructureElement.assertRead("/DISCIPLINE?queryFields=MNEMONIC&queryValues=A__",                                                  45, 45);
            ITUtilStructureElement.assertRead("/DISCIPLINE?statuses=PENDING&queryFields=MNEMONIC&queryValues=A__",                                 10, 10);
            ITUtilStructureElement.assertRead("/DISCIPLINE?statuses=APPROVED&queryFields=MNEMONIC&queryValues=A__",                                20, 20);
            ITUtilStructureElement.assertRead("/DISCIPLINE?statuses=CANCELLED&queryFields=MNEMONIC&queryValues=A__",                                5,  5);
            ITUtilStructureElement.assertRead("/DISCIPLINE?statuses=REJECTED&queryFields=MNEMONIC&queryValues=A__",                                10, 10);
            ITUtilStructureElement.assertRead("/DISCIPLINE?statuses=PENDING&statuses=APPROVED&queryFields=MNEMONIC&queryValues=A__",               30, 30);

            ITUtilStructureElement.assertRead("/DISCIPLINE?deleted=false&queryFields=MNEMONIC&queryValues=A__",                                    30, 30);
            ITUtilStructureElement.assertRead("/DISCIPLINE?deleted=false&statuses=PENDING&queryFields=MNEMONIC&queryValues=A__",                    5,  5);
            ITUtilStructureElement.assertRead("/DISCIPLINE?deleted=false&statuses=APPROVED&queryFields=MNEMONIC&queryValues=A__",                  15, 15);
            ITUtilStructureElement.assertRead("/DISCIPLINE?deleted=false&statuses=CANCELLED&queryFields=MNEMONIC&queryValues=A__",                  5,  5);
            ITUtilStructureElement.assertRead("/DISCIPLINE?deleted=false&statuses=REJECTED&queryFields=MNEMONIC&queryValues=A__",                   5,  5);
            ITUtilStructureElement.assertRead("/DISCIPLINE?deleted=false&statuses=PENDING&statuses=APPROVED&queryFields=MNEMONIC&queryValues=A__", 20, 20);

            ITUtilStructureElement.assertRead("/DISCIPLINE?deleted=true&queryFields=MNEMONIC&queryValues=A__",                                     15, 15);
            ITUtilStructureElement.assertRead("/DISCIPLINE?deleted=true&statuses=PENDING&queryFields=MNEMONIC&queryValues=A__",                     5,  5);
            ITUtilStructureElement.assertRead("/DISCIPLINE?deleted=true&statuses=APPROVED&queryFields=MNEMONIC&queryValues=A__",                    5,  5);
            ITUtilStructureElement.assertRead("/DISCIPLINE?deleted=true&statuses=CANCELLED&queryFields=MNEMONIC&queryValues=A__",                   0,  0);
            ITUtilStructureElement.assertRead("/DISCIPLINE?deleted=true&statuses=REJECTED&queryFields=MNEMONIC&queryValues=A__",                    5,  5);
            ITUtilStructureElement.assertRead("/DISCIPLINE?deleted=true&statuses=PENDING&statuses=APPROVED&queryFields=MNEMONIC&queryValues=A__",  10, 10);

            ITUtilStructureElement.assertRead("/children/DISCIPLINE/" + uuid.toString(), 0, 0);

            ITUtilStructureElement.assertExists(Type.DISCIPLINE, "AA1", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE, "AA2", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE, "AA3", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE, "AA4", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE, "AA5", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE, "AB1", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE, "AB2", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE, "AB3", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE, "AB4", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE, "AB5", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE, "AC1", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE, "AC2", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE, "AC3", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE, "AC4", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE, "AC5", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE, "AD1", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE, "AD2", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE, "AD3", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE, "AD4", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE, "AD5", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE, "AE1", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE, "AE2", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE, "AE3", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE, "AE4", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE, "AE5", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE, "AF1", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE, "AF2", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE, "AF3", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE, "AF4", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE, "AF5", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE, "AG1", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE, "AG2", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE, "AG3", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE, "AG4", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.DISCIPLINE, "AG5", Boolean.TRUE);
        } catch (Exception e) {
            fail();
        }
    }

}
