/*
 * Copyright (C) 2022 European Spallation Source ERIC.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.openepics.names.docker;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.junit.jupiter.api.Assertions.fail;

import java.io.File;
import java.io.IOException;
import java.net.HttpURLConnection;
import java.util.UUID;

import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;
import org.openepics.names.rest.beans.Status;
import org.openepics.names.rest.beans.StructureElement;
import org.openepics.names.rest.beans.Type;
import org.openepics.names.util.EnumUtil.StructureChoice;
import org.testcontainers.containers.DockerComposeContainer;
import org.testcontainers.containers.wait.strategy.Wait;
import org.testcontainers.junit.jupiter.Container;
import org.testcontainers.junit.jupiter.Testcontainers;

/**
 * Integration tests for Naming and PostgreSQL that make use of existing dockerization
 * with docker-compose.yml / Dockerfile.
 *
 * <p>
 * Focus of this class is structures endpoint and subsystem.
 * </p>
 *
 * @author Lars Johansson
 */
@Testcontainers
public class StructuresSubsystemIT {

    // note
    //     StructureElement - when, who, comment - depend on requested or processed
    //         requested, requested by, requested comment
    //         processed, processed by, processed comment
    //     if less than a second between requested and processed, then considered one entry with processed
    //     history
    //         mnemonic path does not make same sense for history
    //         (very) tricky to find mnemonic path for uuid at proper time (history)
    //         therefore empty mnemonic path for history for structure
    //         one history entry if less than one second between requested and processed, otherwise two history entries
    //     attributes for entry for operations - create, update, delete, approve, cancel, reject
    //         some set client side, others set server side
    //         client side
    //             type, uuid, parent uuid, name, mnemonic, description, comment
    //         may be set client side for test purposes
    //     order of methods in test not known
    //         therefore mnemonic values chosen to not interfere with mnemonic values in other methods

    @Container
    public static final DockerComposeContainer<?> ENVIRONMENT =
        new DockerComposeContainer<>(new File("docker-compose-it-db-schema-migration.yml"))
            .waitingFor(ITUtil.NAMING, Wait.forLogMessage(".*Started NamingApplication.*", 1));

    private static UUID systemGroupUuid = null;
    private static UUID systemUuid = null;

    @BeforeAll
    public static void initAll() {
        // init system group, system

        StructureElement structureElement         = null;
        StructureElement approvedStructureElement = null;

        structureElement = new StructureElement(
              Type.SYSTEMGROUP, null, null,
              "name", "Sg", "Sg", 1,
              "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
              null, "test who", "test comment");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        systemGroupUuid = approvedStructureElement.getUuid();

        structureElement = new StructureElement(
                Type.SYSTEM, null, systemGroupUuid,
                "name", "Sys", "Sys", 2,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        systemUuid = approvedStructureElement.getUuid();
    }

    @Test
    public void equivalenceMnemonic() {
        // purpose
        //     test mnemonic equivalence
        //
        // what
        //     read        equivalence mnemonic

        ITUtilStructureElement.assertEquivalence("Sub", "SUB");
    }

    @Test
    public void checkCreate() {
        // purpose
        //     test conditions for create subsystem
        //         not create itself
        //
        // what - combination of
        //     create      create structures
        //     read        exists in structure
        //     read        is valid to create
        //     read        validate create
        //
        // StructureElement attributes
        //     type, name, mnemonic, description, comment

        try {
            StructureElement structureElement = new StructureElement();

            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Cc", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "CC", Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM, "Sg-Sys-Sg",  Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM, "Sg-Sys-Sys", Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM, "Sg-Sys-Cc",  Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM, "Sg-Cc",      Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM, "Sg-CC",      Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM, "Sys-Cc",     Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM, "Sys-CC",     Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM, "SYS-Cc",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM, "SYS-CC",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM, "Sys-Sg",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM, "Sys-SG",     Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM, "Sys-Sys",    Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM, "Sys-SYS",    Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM, "Cc",         Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM, "CC",         Boolean.FALSE);

            ITUtilStructureElement.assertValidate("[{asdf]", StructureChoice.CREATE, HttpURLConnection.HTTP_BAD_REQUEST);
            ITUtilStructureElement.assertCreate("[{asdf]", HttpURLConnection.HTTP_BAD_REQUEST);

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);
            ITUtilStructureElement.assertCreate(structureElement, HttpURLConnection.HTTP_BAD_REQUEST);

            structureElement.setType(Type.SUBSYSTEM);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);
            ITUtilStructureElement.assertCreate(structureElement, HttpURLConnection.HTTP_BAD_REQUEST);

            structureElement.setParent(systemUuid);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);
            ITUtilStructureElement.assertCreate(structureElement, HttpURLConnection.HTTP_BAD_REQUEST);

            structureElement.setName("name");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);
            ITUtilStructureElement.assertCreate(structureElement, HttpURLConnection.HTTP_BAD_REQUEST);

            structureElement.setMnemonic("Cc");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);
            ITUtilStructureElement.assertCreate(structureElement, HttpURLConnection.HTTP_BAD_REQUEST);

            structureElement.setDescription("description");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);
            ITUtilStructureElement.assertCreate(structureElement, HttpURLConnection.HTTP_BAD_REQUEST);

            structureElement.setComment("comment");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.TRUE);

            structureElement.setType(null);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);
        } catch (IOException e) {
            fail();
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void checkCreateMnemonic() {
        // purpose
        //     test conditions for create subsystem
        //         not create itself
        //
        // what - combination of
        //     read        validate create
        //
        // note
        //     mnemonic
        try {
            StructureElement structureElement = new StructureElement();

            structureElement.setType(Type.SUBSYSTEM);
            structureElement.setParent(systemUuid);
            structureElement.setName("name");
            structureElement.setDescription("description");
            structureElement.setComment("comment");

            // mnemonic rules

            structureElement.setMnemonic(null);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);

            structureElement.setMnemonic("");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);

            structureElement.setMnemonic("C");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.TRUE);

            structureElement.setMnemonic("Cc");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.TRUE);

            structureElement.setMnemonic("Ccc");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.TRUE);

            structureElement.setMnemonic("Cccc");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.TRUE);

            structureElement.setMnemonic("Ccccc");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.TRUE);

            structureElement.setMnemonic("Cccccc");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.TRUE);

            structureElement.setMnemonic("Ccccccc");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.TRUE);

            structureElement.setMnemonic("Cccccccc");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.TRUE);

            structureElement.setMnemonic("Ccccccccc");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);

            // mnemonic rules (2)

            structureElement.setMnemonic(" ");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);

            structureElement.setMnemonic("Sub ");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);

            structureElement.setMnemonic("Sub");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.TRUE);

            structureElement.setMnemonic("000");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.TRUE);

            structureElement.setMnemonic("Sub0");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.TRUE);

            structureElement.setMnemonic(":");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);

            structureElement.setMnemonic("Sub:");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);

            structureElement.setMnemonic("Sub:   ");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);

            structureElement.setMnemonic("1");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.TRUE);

            structureElement.setMnemonic("12");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.TRUE);

            structureElement.setMnemonic("123");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.TRUE);

            structureElement.setMnemonic("1234");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.TRUE);

            structureElement.setMnemonic("12345");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.TRUE);

            structureElement.setMnemonic("123456");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.TRUE);

            structureElement.setMnemonic("1234567");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.TRUE);

            structureElement.setMnemonic("12345678");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.TRUE);

            structureElement.setMnemonic("123456789");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void createApprove() {
        // purpose
        //     test create and approve subsystem
        //
        // what - combination of
        //     create      create structures
        //     read        validate create
        //     read        validate approve
        //     patch       approve structures
        //
        // note
        //     create in order to approve

        try {
            StructureElement structureElement         = null;
            StructureElement createdStructureElement  = null;
            StructureElement approvedStructureElement = null;

            structureElement = new StructureElement(
                  Type.SUBSYSTEM, null, systemUuid,
                  "name", "Ca", "Sys-Ca", 3,
                  "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                  null, "test who", "comment");

            ITUtilStructureElement.assertExists         (Type.SUBSYSTEM, structureElement.getMnemonicpath(), Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM, structureElement.getMnemonicpath(), Boolean.TRUE);

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.APPROVE, Boolean.FALSE);

            // create
            createdStructureElement = ITUtilStructureElement.assertCreate(structureElement);
            structureElement = createdStructureElement;

            ITUtilStructureElement.assertExists         (Type.SUBSYSTEM, structureElement.getMnemonicpath(), Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM, structureElement.getMnemonicpath(), Boolean.TRUE);

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.APPROVE, Boolean.TRUE);

            // approve
            approvedStructureElement = ITUtilStructureElement.assertApprove(structureElement);
            structureElement = approvedStructureElement;

            ITUtilStructureElement.assertExists         (Type.SUBSYSTEM, structureElement.getMnemonicpath(), Boolean.TRUE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM, structureElement.getMnemonicpath(), Boolean.FALSE);

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.APPROVE, Boolean.FALSE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void createCancel() {
        // purpose
        //     test create and cancel subsystem
        //
        // what - combination of
        //     create      create structures
        //     read        validate create
        //     read        validate cancel
        //     patch       cancel structures
        //
        // note
        //     create in order to cancel

        try {
            StructureElement structureElement          = null;
            StructureElement createdStructureElement   = null;
            StructureElement cancelledStructureElement = null;

            structureElement = new StructureElement(
                  Type.SUBSYSTEM, null, systemUuid,
                  "name", "Cc", "Sys-Cc", 3,
                  "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                  null, "test who", "comment");

            ITUtilStructureElement.assertExists         (Type.SUBSYSTEM, structureElement.getMnemonicpath(), Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM, structureElement.getMnemonicpath(), Boolean.TRUE);

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CANCEL, Boolean.FALSE);

            // create
            createdStructureElement = ITUtilStructureElement.assertCreate(structureElement);
            structureElement = createdStructureElement;

            ITUtilStructureElement.assertExists         (Type.SUBSYSTEM, structureElement.getMnemonicpath(), Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM, structureElement.getMnemonicpath(), Boolean.TRUE);

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CANCEL, Boolean.TRUE);

            // cancel
            cancelledStructureElement = ITUtilStructureElement.assertCancel(structureElement);
            structureElement = cancelledStructureElement;

            ITUtilStructureElement.assertExists         (Type.SUBSYSTEM, structureElement.getMnemonicpath(), Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM, structureElement.getMnemonicpath(), Boolean.TRUE);

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CANCEL, Boolean.FALSE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void createReject() {
        // purpose
        //     test create and reject subsystem
        //
        // what - combination of
        //     create      create structures
        //     read        validate create
        //     read        validate reject
        //     patch       reject structures
        //
        // note
        //     create in order to reject

        try {
            StructureElement structureElement         = null;
            StructureElement createdStructureElement  = null;
            StructureElement rejectedStructureElement = null;

            structureElement = new StructureElement(
                  Type.SUBSYSTEM, null, systemUuid,
                  "name", "Cr", "Sys-Cr", 3,
                  "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                  null, "test who", "comment");

            ITUtilStructureElement.assertExists         (Type.SUBSYSTEM, structureElement.getMnemonicpath(), Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM, structureElement.getMnemonicpath(), Boolean.TRUE);

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.REJECT, Boolean.FALSE);

            // create
            createdStructureElement = ITUtilStructureElement.assertCreate(structureElement);
            structureElement = createdStructureElement;

            ITUtilStructureElement.assertExists         (Type.SUBSYSTEM, structureElement.getMnemonicpath(), Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM, structureElement.getMnemonicpath(), Boolean.TRUE);

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.REJECT, Boolean.TRUE);

            // reject
            rejectedStructureElement = ITUtilStructureElement.assertReject(structureElement);
            structureElement = rejectedStructureElement;

            ITUtilStructureElement.assertExists         (Type.SUBSYSTEM, structureElement.getMnemonicpath(), Boolean.FALSE);
            ITUtilStructureElement.assertIsValidToCreate(Type.SUBSYSTEM, structureElement.getMnemonicpath(), Boolean.TRUE);

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CREATE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.REJECT, Boolean.FALSE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void checkUpdate() {
        // purpose
        //     test conditions for update subsystem
        //         not update
        //
        // what - combination of
        //     create      create structures
        //     read        validate create
        //     read        validate update
        //     read        validate approve
        //     patch       approve structures
        //
        // note
        //     create, approve in order to update

        try {
            UUID uuid = null;

            StructureElement structureElement         = new StructureElement();
            StructureElement approvedStructureElement = null;

            // create, approve
            structureElement = new StructureElement(
                    Type.SUBSYSTEM, null, systemUuid,
                    "name", "Cu", "Sys-Cu", 3,
                    "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                    null, "test who", "test comment");
            approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
            structureElement = approvedStructureElement;
            uuid = approvedStructureElement.getUuid();

            // validate update

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.TRUE);

            structureElement.setType(null);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.FALSE);

            structureElement.setType(Type.DEVICETYPE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.FALSE);

            structureElement.setType(Type.SUBSYSTEM);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.TRUE);

            structureElement.setUuid(null);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.FALSE);

            structureElement.setUuid(uuid);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.TRUE);

            structureElement.setParent(null);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.FALSE);

            structureElement.setParent(systemUuid);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.TRUE);

            structureElement.setName(null);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.FALSE);

            structureElement.setName("name");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.TRUE);

            structureElement.setMnemonic(null);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.FALSE);

            structureElement.setMnemonic("Cu");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.TRUE);

            structureElement.setDescription(null);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.FALSE);

            structureElement.setDescription("description");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.TRUE);

            structureElement.setComment(null);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.FALSE);

            structureElement.setComment("comment");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.TRUE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void updateApprove() {
        // purpose
        //     test update and approve subsystem
        //
        // what - combination of
        //     create      create structures
        //     read        validate create
        //     read        validate update
        //     read        validate approve
        //     update      update structures
        //     patch       approve structures
        //
        // note
        //     create, approve in order to update, approve

        try {
            StructureElement structureElement         = null;
            StructureElement updatedStructureElement  = null;
            StructureElement approvedStructureElement = null;

            // create, approve
            structureElement = new StructureElement(
                    Type.SUBSYSTEM, null, systemUuid,
                    "name", "Ua", "Sys-Ua", 3,
                    "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                    null, "test who", "test comment");
            approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
            structureElement = approvedStructureElement;

            structureElement.setComment("comment update approve check");

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.APPROVE, Boolean.FALSE);

            // update
            updatedStructureElement = ITUtilStructureElement.assertUpdate(structureElement);
            structureElement = updatedStructureElement;

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.APPROVE, Boolean.TRUE);

            // approve
            approvedStructureElement = ITUtilStructureElement.assertApprove(structureElement);

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.APPROVE, Boolean.FALSE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void updateCancel() {
        // purpose
        //     test update and cancel subsystem
        //
        // what - combination of
        //     create      create structures
        //     read        validate create
        //     read        validate update
        //     read        validate approve
        //     read        validate cancel
        //     update      update structures
        //     patch       approve structures
        //     patch       cancel structures
        //
        // note
        //     create, approve in order to update, cancel

        try {
            StructureElement structureElement          = null;
            StructureElement updatedStructureElement   = null;
            StructureElement approvedStructureElement  = null;
            StructureElement cancelledStructureElement = null;

            // create, approve
            structureElement = new StructureElement(
                    Type.SUBSYSTEM, null, systemUuid,
                    "name", "Uc", "Sys-Uc", 3,
                    "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                    null, "test who", "test comment");
            approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
            structureElement = approvedStructureElement;

            structureElement.setComment("comment update cancel check");

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CANCEL, Boolean.FALSE);

            // update
            updatedStructureElement = ITUtilStructureElement.assertUpdate(structureElement);
            structureElement = updatedStructureElement;

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CANCEL, Boolean.TRUE);

            // cancel
            cancelledStructureElement = ITUtilStructureElement.assertCancel(structureElement);
            structureElement = cancelledStructureElement;

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CANCEL, Boolean.FALSE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void updateReject() {
        // purpose
        //     test update and reject subsystem
        //
        // what - combination of
        //     create      create structures
        //     read        validate create
        //     read        validate update
        //     read        validate approve
        //     read        validate reject
        //     update      update structures
        //     patch       approve structures
        //     patch       reject structures
        //
        // note
        //     create, approve in order to update, reject

        try {
            StructureElement structureElement         = null;
            StructureElement updatedStructureElement  = null;
            StructureElement approvedStructureElement = null;
            StructureElement rejectedStructureElement = null;

            // create, approve
            structureElement = new StructureElement(
                    Type.SUBSYSTEM, null, systemUuid,
                    "name", "Ur", "Sys-Ur", 3,
                    "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                    null, "test who", "test comment");
            approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
            structureElement = approvedStructureElement;

            structureElement.setComment("comment update reject check");

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.REJECT, Boolean.FALSE);

            // update
            updatedStructureElement = ITUtilStructureElement.assertUpdate(structureElement);
            structureElement = updatedStructureElement;

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.REJECT, Boolean.TRUE);

            // reject
            rejectedStructureElement = ITUtilStructureElement.assertReject(structureElement);
            structureElement = rejectedStructureElement;

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.UPDATE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.REJECT, Boolean.FALSE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void checkDelete() {
        // purpose
        //     test conditions for delete subsystem
        //         not delete
        //
        // what - combination of
        //     create      create structures
        //     read        validate create
        //     read        validate delete
        //     read        validate approve
        //     patch       approve structures
        //
        // note
        //     create, approve in order to delete

        try {
            UUID uuid = null;

            StructureElement   structureElement         = new StructureElement();
            StructureElement   approvedStructureElement = null;

            // create, approve
            structureElement = new StructureElement(
                    Type.SUBSYSTEM, null, systemUuid,
                    "name", "Cd", "Sys-Cd", 3,
                    "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                    null, "test who", "test comment");
            approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
            structureElement = approvedStructureElement;
            uuid = approvedStructureElement.getUuid();

            // validate delete

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.TRUE);

            structureElement.setType(null);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.FALSE);

            structureElement.setType(Type.DEVICETYPE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.FALSE);

            structureElement.setType(Type.SUBSYSTEM);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.TRUE);

            structureElement.setUuid(null);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.FALSE);

            structureElement.setUuid(uuid);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.TRUE);

            structureElement.setParent(null);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.FALSE);

            structureElement.setParent(systemGroupUuid);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.TRUE);

            structureElement.setName(null);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.FALSE);

            structureElement.setName("name");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.TRUE);

            structureElement.setMnemonic(null);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.FALSE);

            structureElement.setMnemonic("Cd");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.TRUE);

            structureElement.setDescription(null);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.FALSE);

            structureElement.setDescription("description");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.TRUE);

            structureElement.setComment(null);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.FALSE);

            structureElement.setComment("comment");
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.TRUE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void deleteApprove() {
        // purpose
        //     test delete and approve subsystem
        //
        // what - combination of
        //     create      create structures
        //     read        validate create
        //     read        validate delete
        //     read        validate approve
        //     delete      delete structures
        //     patch       approve structures
        //
        // note
        //     create, approve in order to delete, approve

        try {
            StructureElement structureElement         = null;
            StructureElement deletedStructureElement  = null;
            StructureElement approvedStructureElement = null;

            // create, approve
            structureElement = new StructureElement(
                    Type.SUBSYSTEM, null, systemUuid,
                    "name", "Da", "Sys-Da", 3,
                    "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                    null, "test who", "test comment");
            approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
            structureElement = approvedStructureElement;

            structureElement.setComment("comment delete approve check");

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE,  Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.APPROVE, Boolean.FALSE);

            // delete
            structureElement.setDeleted(Boolean.TRUE);
            deletedStructureElement = ITUtilStructureElement.assertDelete(structureElement);
            structureElement = deletedStructureElement;

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.APPROVE, Boolean.TRUE);

            // approve
            approvedStructureElement = ITUtilStructureElement.assertApprove(structureElement);

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE,  Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.APPROVE, Boolean.FALSE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void deleteCancel() {
        // purpose
        //     test delete and cancel subsystem
        //
        // what - combination of
        //     create      create structures
        //     read        validate create
        //     read        validate delete
        //     read        validate approve
        //     read        validate cancel
        //     update      delete structures
        //     patch       approve structures
        //     patch       cancel structures
        //
        // note
        //     create, approve in order to delete, cancel

        try {
            StructureElement structureElement          = null;
            StructureElement deletedStructureElement   = null;
            StructureElement approvedStructureElement  = null;
            StructureElement cancelledStructureElement = null;

            // create, approve
            structureElement = new StructureElement(
                    Type.SUBSYSTEM, null, systemUuid,
                    "name", "Dc", "Sys-Dc", 3,
                    "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                    null, "test who", "test comment");
            approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
            structureElement = approvedStructureElement;

            structureElement.setComment("comment update delete check");

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CANCEL, Boolean.FALSE);

            // delete
            deletedStructureElement = ITUtilStructureElement.assertDelete(structureElement);
            structureElement = deletedStructureElement;

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CANCEL, Boolean.TRUE);

            // cancel
            cancelledStructureElement = ITUtilStructureElement.assertCancel(structureElement);
            structureElement = cancelledStructureElement;

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.CANCEL, Boolean.FALSE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void deleteReject() {
        // purpose
        //     test delete and reject subsystem
        //
        // what - combination of
        //     create      create structures
        //     read        validate create
        //     read        validate delete
        //     read        validate approve
        //     read        validate reject
        //     update      delete structures
        //     patch       approve structures
        //     patch       reject structures
        //
        // note
        //     create, approve in order to delete, reject

        try {
            StructureElement structureElement         = null;
            StructureElement deletedStructureElement  = null;
            StructureElement approvedStructureElement = null;
            StructureElement rejectedStructureElement = null;

            // create, approve
            structureElement = new StructureElement(
                    Type.SUBSYSTEM, null, systemUuid,
                    "name", "Dr", "Sys-Dr", 3,
                    "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                    null, "test who", "test comment");
            approvedStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
            structureElement = approvedStructureElement;

            structureElement.setComment("comment delete reject check");

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.REJECT, Boolean.FALSE);

            // delete
            deletedStructureElement = ITUtilStructureElement.assertDelete(structureElement);
            structureElement = deletedStructureElement;

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.FALSE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.REJECT, Boolean.TRUE);

            // reject
            rejectedStructureElement = ITUtilStructureElement.assertReject(structureElement);
            structureElement = rejectedStructureElement;

            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.DELETE, Boolean.TRUE);
            ITUtilStructureElement.assertValidate(structureElement, StructureChoice.REJECT, Boolean.FALSE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void readSearchHistoryApprove() {
        // purpose
        //     test read subsystem in various ways for create, approve
        //         search
        //         latest
        //         history
        //
        // note
        //     create (and more) to read (with content)

        try {
            StructureElement   structureElement         = null;
            StructureElement   createdStructureElement  = null;
            StructureElement   approvedStructureElement = null;
            int length = 0;

            structureElement = new StructureElement(
                  Type.SUBSYSTEM, null, systemUuid,
                  "name", "Rsha", "Sys-Rsha", 3,
                  "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                  null, "test who", "comment");

            // create
            createdStructureElement = ITUtilStructureElement.assertCreate(structureElement);
            structureElement = createdStructureElement;

            // read (1)
            length = ITUtilStructureElement.assertRead("/SUBSYSTEM", -1, 1);

            ITUtilStructureElement.assertRead("/SUBSYSTEM?statuses=PENDING",   -1, 1);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?statuses=APPROVED",  -1, 0);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?statuses=CANCELLED", -1, 0);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?statuses=REJECTED",  -1, 0);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?statuses=PENDING&queryFields=MNEMONIC&queryValues=Rsha",   1, 1);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?statuses=APPROVED&queryFields=MNEMONIC&queryValues=Rsha",  0, 0);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?statuses=CANCELLED&queryFields=MNEMONIC&queryValues=Rsha", 0, 0);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?statuses=REJECTED&queryFields=MNEMONIC&queryValues=Rsha",  0, 0);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?queryFields=UUID&queryValues=" + createdStructureElement.getUuid().toString(), 1, 1);

            ITUtilStructureElement.assertRead("/mnemonic/Sg",              1, 1);
            ITUtilStructureElement.assertRead("/mnemonic/Sys",             1, 1);
            ITUtilStructureElement.assertRead("/mnemonic/Rsha",            0, 0);
            ITUtilStructureElement.assertRead("/mnemonic/Sg-Sys",          0, 0);
            ITUtilStructureElement.assertRead("/mnemonic/Sys-Rsha",        0, 0);
            ITUtilStructureElement.assertRead("/mnemonic/Sg-Sys-Rsha",     0, 0);

            ITUtilStructureElement.assertRead("/mnemonicpath/Sg",          1, 1);
            ITUtilStructureElement.assertRead("/mnemonicpath/Sys",         1, 1);
            ITUtilStructureElement.assertRead("/mnemonicpath/Rsha",        0, 0);
            ITUtilStructureElement.assertRead("/mnemonicpath/Sg-Sys",      0, 0);
            ITUtilStructureElement.assertRead("/mnemonicpath/Sys-Rsha",    0, 0);
            ITUtilStructureElement.assertRead("/mnemonicpath/Sg-Sys-Rsha", 0, 0);

            ITUtilStructureElement.assertRead("/history/" + createdStructureElement.getUuid().toString(), -1, 1);

            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Rsha",        Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sys-Rsha",    Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sg-Sys-Rsha", Boolean.FALSE);

            // approve
            approvedStructureElement = ITUtilStructureElement.assertApprove(structureElement);

            // read (2)
            assertEquals(length, ITUtilStructureElement.assertRead("/SUBSYSTEM", -1, 1));

            ITUtilStructureElement.assertRead("/SUBSYSTEM?statuses=PENDING",   -1, 0);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?statuses=APPROVED",  -1, 1);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?statuses=CANCELLED", -1, 0);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?statuses=REJECTED",  -1, 0);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?statuses=PENDING&queryFields=MNEMONIC&queryValues=Rsha",   0, 0);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?statuses=APPROVED&queryFields=MNEMONIC&queryValues=Rsha",  1, 1);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?statuses=CANCELLED&queryFields=MNEMONIC&queryValues=Rsha", 0, 0);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?statuses=REJECTED&queryFields=MNEMONIC&queryValues=Rsha",  0, 0);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?queryFields=UUID&queryValues=" + approvedStructureElement.getUuid().toString(), 1, 1);

            ITUtilStructureElement.assertRead("/mnemonic/Sg",              1, 1);
            ITUtilStructureElement.assertRead("/mnemonic/Sys",             1, 1);
            ITUtilStructureElement.assertRead("/mnemonic/Rsha",            1, 1);
            ITUtilStructureElement.assertRead("/mnemonic/Sg-Sys",          0, 0);
            ITUtilStructureElement.assertRead("/mnemonic/Sys-Rsha",        0, 0);
            ITUtilStructureElement.assertRead("/mnemonic/Sg-Sys-Rsha",     0, 0);

            ITUtilStructureElement.assertRead("/mnemonicpath/Sg",          1, 1);
            ITUtilStructureElement.assertRead("/mnemonicpath/Sys",         1, 1);
            ITUtilStructureElement.assertRead("/mnemonicpath/Rsha",        0, 0);
            ITUtilStructureElement.assertRead("/mnemonicpath/Sg-Sys",      0, 0);
            ITUtilStructureElement.assertRead("/mnemonicpath/Sys-Rsha",    1, 1);
            ITUtilStructureElement.assertRead("/mnemonicpath/Sg-Sys-Rsha", 0, 0);

            ITUtilStructureElement.assertRead("/history/" + approvedStructureElement.getUuid().toString(), -1, 1);

            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Rsha",        Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sys-Rsha",    Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sg-Sys-Rsha", Boolean.FALSE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void readSearchHistoryCancel() {
        // purpose
        //     test read subsystem in various ways for create, cancel
        //         search
        //         latest
        //         history
        //
        // note
        //     create (and more) to read (with content)

        try {
            StructureElement   structureElement          = null;
            StructureElement   createdStructureElement   = null;
            StructureElement   cancelledStructureElement = null;
            int length = 0;

            structureElement = new StructureElement(
                  Type.SUBSYSTEM, null, systemUuid,
                  "name", "Rshc", "Sys-Rshc", 3,
                  "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                  null, "test who", "comment");

            // create
            createdStructureElement = ITUtilStructureElement.assertCreate(structureElement);
            structureElement = createdStructureElement;

            // read (1)
            length = ITUtilStructureElement.assertRead("/SUBSYSTEM", -1, 1);

            ITUtilStructureElement.assertRead("/SUBSYSTEM?statuses=PENDING",   -1, 1);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?statuses=APPROVED",  -1, 0);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?statuses=CANCELLED", -1, 0);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?statuses=REJECTED",  -1, 0);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?statuses=PENDING&queryFields=MNEMONIC&queryValues=Rshc",   1, 1);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?statuses=APPROVED&queryFields=MNEMONIC&queryValues=Rshc",  0, 0);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?statuses=CANCELLED&queryFields=MNEMONIC&queryValues=Rshc", 0, 0);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?statuses=REJECTED&queryFields=MNEMONIC&queryValues=Rshc",  0, 0);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?queryFields=UUID&queryValues=" + createdStructureElement.getUuid().toString(), 1, 1);

            ITUtilStructureElement.assertRead("/mnemonic/Sg",              1, 1);
            ITUtilStructureElement.assertRead("/mnemonic/Sys",             1, 1);
            ITUtilStructureElement.assertRead("/mnemonic/Rshc",            0, 0);
            ITUtilStructureElement.assertRead("/mnemonic/Sg-Sys",          0, 0);
            ITUtilStructureElement.assertRead("/mnemonic/Sys-Rshc",        0, 0);
            ITUtilStructureElement.assertRead("/mnemonic/Sg-Sys-Rshc",     0, 0);

            ITUtilStructureElement.assertRead("/mnemonicpath/Sg",          1, 1);
            ITUtilStructureElement.assertRead("/mnemonicpath/Sys",         1, 1);
            ITUtilStructureElement.assertRead("/mnemonicpath/Rshc",        0, 0);
            ITUtilStructureElement.assertRead("/mnemonicpath/Sg-Sys",      0, 0);
            ITUtilStructureElement.assertRead("/mnemonicpath/Sys-Rshc",    0, 0);
            ITUtilStructureElement.assertRead("/mnemonicpath/Sg-Sys-Rshc", 0, 0);

            ITUtilStructureElement.assertRead("/history/" + createdStructureElement.getUuid().toString(), -1, 1);

            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Rshc",        Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sys-Rshc",    Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sg-Sys-Rshc", Boolean.FALSE);

            // cancel
            cancelledStructureElement = ITUtilStructureElement.assertCancel(structureElement);

            // read (2)
            assertEquals(length, ITUtilStructureElement.assertRead("/SUBSYSTEM", -1, 1));

            ITUtilStructureElement.assertRead("/SUBSYSTEM?statuses=PENDING",   -1, 0);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?statuses=APPROVED",  -1, 0);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?statuses=CANCELLED", -1, 1);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?statuses=REJECTED",  -1, 0);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?statuses=PENDING&queryFields=MNEMONIC&queryValues=Rshc",   0, 0);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?statuses=APPROVED&queryFields=MNEMONIC&queryValues=Rshc",  0, 0);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?statuses=CANCELLED&queryFields=MNEMONIC&queryValues=Rshc", 1, 1);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?statuses=REJECTED&queryFields=MNEMONIC&queryValues=Rshc",  0, 0);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?queryFields=UUID&queryValues=" + cancelledStructureElement.getUuid().toString(), 1, 1);

            ITUtilStructureElement.assertRead("/mnemonic/Sg",              1, 1);
            ITUtilStructureElement.assertRead("/mnemonic/Sys",             1, 1);
            ITUtilStructureElement.assertRead("/mnemonic/Rshc",            0, 0);
            ITUtilStructureElement.assertRead("/mnemonic/Sg-Sys",          0, 0);
            ITUtilStructureElement.assertRead("/mnemonic/Sys-Rshc",        0, 0);
            ITUtilStructureElement.assertRead("/mnemonic/Sg-Sys-Rshc",     0, 0);

            ITUtilStructureElement.assertRead("/mnemonicpath/Sg",          1, 1);
            ITUtilStructureElement.assertRead("/mnemonicpath/Sys",         1, 1);
            ITUtilStructureElement.assertRead("/mnemonicpath/Rshc",        0, 0);
            ITUtilStructureElement.assertRead("/mnemonicpath/Sg-Sys",      0, 0);
            ITUtilStructureElement.assertRead("/mnemonicpath/Sys-Rshc",    0, 0);
            ITUtilStructureElement.assertRead("/mnemonicpath/Sg-Sys-Rshc", 0, 0);

            ITUtilStructureElement.assertRead("/history/" + cancelledStructureElement.getUuid().toString(), -1, 1);

            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Rshc",        Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sys-Rshc",    Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sg-Sys-Rshc", Boolean.FALSE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void readSearchHistoryReject() {
        // purpose
        //     test read subsystem in various ways for create, reject
        //         search
        //         latest
        //         history
        //
        // note
        //     create (and more) to read (with content)

        try {
            StructureElement   structureElement         = null;
            StructureElement   createdStructureElement  = null;
            StructureElement   rejectedStructureElement = null;
            int length = 0;

            structureElement = new StructureElement(
                  Type.SUBSYSTEM, null, systemUuid,
                  "name", "Rshr", "Sys-Rshr", 3,
                  "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                  null, "test who", "comment");

            // create
            createdStructureElement = ITUtilStructureElement.assertCreate(structureElement);
            structureElement = createdStructureElement;

            // read (1)
            length = ITUtilStructureElement.assertRead("/SUBSYSTEM", -1, 1);

            ITUtilStructureElement.assertRead("/SUBSYSTEM?statuses=PENDING",   -1, 1);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?statuses=APPROVED",  -1, 0);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?statuses=CANCELLED", -1, 0);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?statuses=REJECTED",  -1, 0);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?statuses=PENDING&queryFields=MNEMONIC&queryValues=Rshr",   1, 1);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?statuses=APPROVED&queryFields=MNEMONIC&queryValues=Rshr",  0, 0);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?statuses=CANCELLED&queryFields=MNEMONIC&queryValues=Rshr", 0, 0);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?statuses=REJECTED&queryFields=MNEMONIC&queryValues=Rshr",  0, 0);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?queryFields=UUID&queryValues=" + createdStructureElement.getUuid().toString(), 1, 1);

            ITUtilStructureElement.assertRead("/mnemonic/Sg",              1, 1);
            ITUtilStructureElement.assertRead("/mnemonic/Sys",             1, 1);
            ITUtilStructureElement.assertRead("/mnemonic/Rshr",            0, 0);
            ITUtilStructureElement.assertRead("/mnemonic/Sg-Sys",          0, 0);
            ITUtilStructureElement.assertRead("/mnemonic/Sys-Rshr",        0, 0);
            ITUtilStructureElement.assertRead("/mnemonic/Sg-Sys-Rshr",     0, 0);

            ITUtilStructureElement.assertRead("/mnemonicpath/Sg",          1, 1);
            ITUtilStructureElement.assertRead("/mnemonicpath/Sys",         1, 1);
            ITUtilStructureElement.assertRead("/mnemonicpath/Rshr",        0, 0);
            ITUtilStructureElement.assertRead("/mnemonicpath/Sg-Sys",      0, 0);
            ITUtilStructureElement.assertRead("/mnemonicpath/Sys-Rshr",    0, 0);
            ITUtilStructureElement.assertRead("/mnemonicpath/Sg-Sys-Rshr", 0, 0);

            ITUtilStructureElement.assertRead("/history/" + createdStructureElement.getUuid().toString(), -1, 1);

            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Rshr",        Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sys-Rshr",    Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sg-Sys-Rshr", Boolean.FALSE);

            // reject
            rejectedStructureElement = ITUtilStructureElement.assertReject(structureElement);

            // read (2)
            assertEquals(length, ITUtilStructureElement.assertRead("/SUBSYSTEM", -1, 1));

            ITUtilStructureElement.assertRead("/SUBSYSTEM?statuses=PENDING",   -1, 0);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?statuses=APPROVED",  -1, 0);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?statuses=CANCELLED", -1, 0);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?statuses=REJECTED",  -1, 1);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?statuses=PENDING&queryFields=MNEMONIC&queryValues=Rshr",   0, 0);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?statuses=APPROVED&queryFields=MNEMONIC&queryValues=Rshr",  0, 0);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?statuses=CANCELLED&queryFields=MNEMONIC&queryValues=Rshr", 0, 0);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?statuses=REJECTED&queryFields=MNEMONIC&queryValues=Rshr",  1, 1);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?queryFields=UUID&queryValues=" + rejectedStructureElement.getUuid().toString(), 1, 1);

            ITUtilStructureElement.assertRead("/mnemonic/Sg",              1, 1);
            ITUtilStructureElement.assertRead("/mnemonic/Sys",             1, 1);
            ITUtilStructureElement.assertRead("/mnemonic/Rshr",            0, 0);
            ITUtilStructureElement.assertRead("/mnemonic/Sg-Sys",          0, 0);
            ITUtilStructureElement.assertRead("/mnemonic/Sys-Rshr",        0, 0);
            ITUtilStructureElement.assertRead("/mnemonic/Sg-Sys-Rshr",     0, 0);

            ITUtilStructureElement.assertRead("/mnemonicpath/Sg",          1, 1);
            ITUtilStructureElement.assertRead("/mnemonicpath/Sys",         1, 1);
            ITUtilStructureElement.assertRead("/mnemonicpath/Rshr",        0, 0);
            ITUtilStructureElement.assertRead("/mnemonicpath/Sg-Sys",      0, 0);
            ITUtilStructureElement.assertRead("/mnemonicpath/Sys-Rshr",    0, 0);
            ITUtilStructureElement.assertRead("/mnemonicpath/Sg-Sys-Rshr", 0, 0);

            ITUtilStructureElement.assertRead("/history/" + rejectedStructureElement.getUuid().toString(), -1, 1);

            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Rshr",        Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sys-Rshr",    Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sg-Sys-Rshr", Boolean.FALSE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void readSearchHistoryStatusLatest1() {
        // purpose
        //     test read subsystem in various ways
        //         status
        //         latest
        //       ( deleted )
        //         history
        //
        // what
        //     entries with different statuses
        //
        // note
        //     create (and more) to read (with content)
        //     4 different lines  of uuid with combinations of status and latest
        //         1 entry for each line of uuid
        //         in particular different statuses for last entry in lines of uuid
        //         mnemonic - P, C, R, A (pending, cancelled, rejected, approved)
        //
        // note!
        //     not only
        //         exclude APPROVED and not latest
        //     not only
        //         exclude earlier than APPROVED and latest (considered history)
        //         include PENDING and not latest           (considered future)
        //     instead
        //         exclude earlier than APPROVED and latest (considered history)
        //       ( include if latest )
        //         include if latest not exists             (considered present + future)

        StructureElement structureElement         = null;
        StructureElement responseStructureElement = null;
        UUID uuid, uuid2, uuid3, uuid4 = null;

        // a number of entries for which the last entry has status PENDING
        structureElement = new StructureElement(
                Type.SUBSYSTEM, null, systemUuid,
                "name", "P1", "Sys-P1", 3,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment 1");
        responseStructureElement = ITUtilStructureElement.assertCreate(structureElement);
        uuid = responseStructureElement.getUuid();

        // a number of entries for which the last entry has status CANCELLED
        structureElement = new StructureElement(
                Type.SUBSYSTEM, null, systemUuid,
                "name", "C1", "Sys-C1", 3,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment 1");
        responseStructureElement = ITUtilStructureElement.assertCreateCancel(structureElement);
        uuid2 = responseStructureElement.getUuid();

        // a number of entries for which the last entry has status REJECTED
        structureElement = new StructureElement(
                Type.SUBSYSTEM, null, systemUuid,
                "name", "R1", "Sys-R1", 3,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment 1");
        responseStructureElement = ITUtilStructureElement.assertCreateReject(structureElement);
        uuid3 = responseStructureElement.getUuid();

        // a number of entries for which the last entry has status APPROVED
        structureElement = new StructureElement(
                Type.SUBSYSTEM, null, systemUuid,
                "name", "A1", "Sys-A1", 3,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment 1");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        uuid4 = responseStructureElement.getUuid();

        try {
            // from first structure element
            assertNotNull(uuid);

            // note
            //     exclude earlier than APPROVED and latest (considered history)
            //   ( include if latest )
            //     include if latest not exists             (considered present + future)

            ITUtilStructureElement.assertRead("/SUBSYSTEM?queryFields=UUID&queryValues=" + uuid.toString(),  1, 1);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?queryFields=UUID&queryValues=" + uuid2.toString(), 1, 1);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?queryFields=UUID&queryValues=" + uuid3.toString(), 1, 1);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?queryFields=UUID&queryValues=" + uuid4.toString(), 1, 1);

            ITUtilStructureElement.assertRead("/mnemonic/P1", 0, 0);
            ITUtilStructureElement.assertRead("/mnemonic/C1", 0, 0);
            ITUtilStructureElement.assertRead("/mnemonic/R1", 0, 0);
            ITUtilStructureElement.assertRead("/mnemonic/A1", 1, 1);

            // normally expected number of entries are 1, 2, 2, 2
            // since within 1 second from requested to processed, expected number of entries 1, 1, 1, 1
            ITUtilStructureElement.assertRead("/history/" + uuid.toString(),  1, 1);
            ITUtilStructureElement.assertRead("/history/" + uuid2.toString(), 1, 1);
            ITUtilStructureElement.assertRead("/history/" + uuid3.toString(), 1, 1);
            ITUtilStructureElement.assertRead("/history/" + uuid4.toString(), 1, 1);

            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "P1",        Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "C1",        Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "R1",        Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "A1",        Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sys-P1",    Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sys-C1",    Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sys-R1",    Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sys-A1",    Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sg-Sys-P1", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sg-Sys-C1", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sg-Sys-R1", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sg-Sys-A1", Boolean.FALSE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void readSearchHistoryStatusLatest9() {
        // purpose
        //     test read subsystem in various ways
        //         status
        //         latest
        //       ( deleted )
        //         history
        //
        // what
        //     entries with different statuses
        //
        // note
        //     create (and more) to read (with content)
        //     4 different lines  of uuid with combinations of status and latest
        //         9 entries for each line of uuid
        //         in particular different statuses for last entry in lines of uuid
        //         mnemonic - P, C, R, A (pending, cancelled, rejected, approved)
        //
        // note!
        //     not only
        //         exclude APPROVED and not latest
        //     not only
        //         exclude earlier than APPROVED and latest (considered history)
        //         include PENDING and not latest           (considered future)
        //     instead
        //         exclude earlier than APPROVED and latest (considered history)
        //       ( include if latest )
        //         include if latest not exists             (considered present + future)

        StructureElement structureElement         = null;
        StructureElement responseStructureElement = null;
        UUID uuid, uuid2, uuid3, uuid4 = null;

        // a number of entries for which the last entry has status PENDING
        structureElement = new StructureElement(
                Type.SUBSYSTEM, null, systemUuid,
                "name", "P9", "Sys-P9", 3,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment 1");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        uuid = responseStructureElement.getUuid();

        responseStructureElement.setComment("test comment 2");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("test comment 3");
        responseStructureElement = ITUtilStructureElement.assertUpdateCancel(responseStructureElement);
        responseStructureElement.setComment("test comment 4");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("test comment 5");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("test comment 6");
        responseStructureElement = ITUtilStructureElement.assertUpdateReject(responseStructureElement);
        responseStructureElement.setComment("test comment 7");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("test comment 8");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("test comment 9");
        responseStructureElement = ITUtilStructureElement.assertUpdate(responseStructureElement);

        // a number of entries for which the last entry has status CANCELLED
        structureElement = new StructureElement(
                Type.SUBSYSTEM, null, systemUuid,
                "name", "C9", "Sys-C9", 3,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment 1");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        uuid2 = responseStructureElement.getUuid();

        responseStructureElement.setComment("test comment 2");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("test comment 3");
        responseStructureElement = ITUtilStructureElement.assertUpdateCancel(responseStructureElement);
        responseStructureElement.setComment("test comment 4");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("test comment 5");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("test comment 6");
        responseStructureElement = ITUtilStructureElement.assertUpdateReject(responseStructureElement);
        responseStructureElement.setComment("test comment 7");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("test comment 8");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("test comment 9");
        responseStructureElement = ITUtilStructureElement.assertUpdateCancel(responseStructureElement);

        // a number of entries for which the last entry has status REJECTED
        structureElement = new StructureElement(
                Type.SUBSYSTEM, null, systemUuid,
                "name", "R9", "Sys-R9", 3,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment 1");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        uuid3 = responseStructureElement.getUuid();

        responseStructureElement.setComment("test comment 2");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("test comment 3");
        responseStructureElement = ITUtilStructureElement.assertUpdateCancel(responseStructureElement);
        responseStructureElement.setComment("test comment 4");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("test comment 5");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("test comment 6");
        responseStructureElement = ITUtilStructureElement.assertUpdateReject(responseStructureElement);
        responseStructureElement.setComment("test comment 7");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("test comment 8");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("test comment 9");
        responseStructureElement = ITUtilStructureElement.assertUpdateReject(responseStructureElement);

        // a number of entries for which the last entry has status APPROVED
        structureElement = new StructureElement(
                Type.SUBSYSTEM, null, systemUuid,
                "name", "A9", "Sys-A9", 3,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment 1");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        uuid4 = responseStructureElement.getUuid();

        responseStructureElement.setComment("test comment 2");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("test comment 3");
        responseStructureElement = ITUtilStructureElement.assertUpdateCancel(responseStructureElement);
        responseStructureElement.setComment("test comment 4");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("test comment 5");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("test comment 6");
        responseStructureElement = ITUtilStructureElement.assertUpdateReject(responseStructureElement);
        responseStructureElement.setComment("test comment 7");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("test comment 8");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);
        responseStructureElement.setComment("test comment 9");
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(responseStructureElement);

        try {
            // from first structure element
            assertNotNull(uuid);

            // note
            //     exclude earlier than APPROVED and latest (considered history)
            //   ( include if latest )
            //     include if latest not exists             (considered present + future)

            ITUtilStructureElement.assertRead("/SUBSYSTEM?queryFields=UUID&queryValues=" + uuid.toString(),  2, 2);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?queryFields=UUID&queryValues=" + uuid2.toString(), 2, 2);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?queryFields=UUID&queryValues=" + uuid3.toString(), 2, 2);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?queryFields=UUID&queryValues=" + uuid4.toString(), 1, 1);

            ITUtilStructureElement.assertRead("/mnemonic/P9", 1, 1);
            ITUtilStructureElement.assertRead("/mnemonic/C9", 1, 1);
            ITUtilStructureElement.assertRead("/mnemonic/R9", 1, 1);
            ITUtilStructureElement.assertRead("/mnemonic/A9", 1, 1);

            // normally expected number of entries are 17, 18, 18, 18
            // since within 1 second from requested to processed, expected number of entries 9, 9, 9, 9
            ITUtilStructureElement.assertRead("/history/" + uuid.toString(),  9, 9);
            ITUtilStructureElement.assertRead("/history/" + uuid2.toString(), 9, 9);
            ITUtilStructureElement.assertRead("/history/" + uuid3.toString(), 9, 9);
            ITUtilStructureElement.assertRead("/history/" + uuid4.toString(), 9, 9);

            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "P9",        Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "C9",        Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "R9",        Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "A9",        Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sys-P9",    Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sys-C9",    Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sys-R9",    Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sys-A9",    Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sg-Sys-P9", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sg-Sys-C9", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sg-Sys-R9", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sg-Sys-A9", Boolean.FALSE);
        } catch (Exception e) {
            fail();
        }
    }

    @Test
    public void readSearchStatusDeletedChildren() {
        // purpose
        //     test read subsystem in various ways
        //         status
        //       ( latest )
        //         deleted
        //         children
        //
        // what
        //     entries with different statuses
        //
        // note
        //     create (and more) to read (with content)
        //     querying for Status.APPROVED means latest approved

        StructureElement structureElement         = null;
        StructureElement responseStructureElement = null;
        UUID uuid = null;

        structureElement = new StructureElement(
                Type.SUBSYSTEM, null, systemUuid,
                "name", "AA1", "Sys-AA1", 3,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        uuid = responseStructureElement.getUuid();

        structureElement = new StructureElement(
                Type.SUBSYSTEM, null, systemUuid,
                "name", "AA2", "Sys-AA2", 3,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);

        structureElement = new StructureElement(
                Type.SUBSYSTEM, null, systemUuid,
                "name", "AA3", "Sys-AA3", 3,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);

        structureElement = new StructureElement(
                Type.SUBSYSTEM, null, systemUuid,
                "name", "AA4", "Sys-AA4", 3,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);

        structureElement = new StructureElement(
                Type.SUBSYSTEM, null, systemUuid,
                "name", "AA5", "Sys-AA5", 3,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);

        structureElement = new StructureElement(
                Type.SUBSYSTEM, null, systemUuid,
                "name", "AB1", "Sys-AB1", 3,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateCancel(structureElement);

        structureElement = new StructureElement(
                Type.SUBSYSTEM, null, systemUuid,
                "name", "AB2", "Sys-AB2", 3,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateCancel(structureElement);

        structureElement = new StructureElement(
                Type.SUBSYSTEM, null, systemUuid,
                "name", "AB3", "Sys-AB3", 3,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateCancel(structureElement);

        structureElement = new StructureElement(
                Type.SUBSYSTEM, null, systemUuid,
                "name", "AB4", "Sys-AB4", 3,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateCancel(structureElement);

        structureElement = new StructureElement(
                Type.SUBSYSTEM, null, systemUuid,
                "name", "AB5", "Sys-AB5", 3,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateCancel(structureElement);

        structureElement = new StructureElement(
                Type.SUBSYSTEM, null, systemUuid,
                "name", "AC1", "Sys-AC1", 3,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateReject(structureElement);

        structureElement = new StructureElement(
                Type.SUBSYSTEM, null, systemUuid,
                "name", "AC2", "Sys-AC2", 3,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateReject(structureElement);

        structureElement = new StructureElement(
                Type.SUBSYSTEM, null, systemUuid,
                "name", "AC3", "Sys-AC3", 3,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateReject(structureElement);

        structureElement = new StructureElement(
                Type.SUBSYSTEM, null, systemUuid,
                "name", "AC4", "Sys-AC4", 3,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateReject(structureElement);

        structureElement = new StructureElement(
                Type.SUBSYSTEM, null, systemUuid,
                "name", "AC5", "Sys-AC5", 3,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateReject(structureElement);

        structureElement = new StructureElement(
                Type.SUBSYSTEM, null, systemUuid,
                "name", "AD1", "Sys-AD1", 3,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreate(structureElement);

        structureElement = new StructureElement(
                Type.SUBSYSTEM, null, systemUuid,
                "name", "AD2", "Sys-AD2", 3,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreate(structureElement);

        structureElement = new StructureElement(
                Type.SUBSYSTEM, null, systemUuid,
                "name", "AD3", "Sys-AD3", 3,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreate(structureElement);

        structureElement = new StructureElement(
                Type.SUBSYSTEM, null, systemUuid,
                "name", "AD4", "Sys-AD4", 3,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreate(structureElement);

        structureElement = new StructureElement(
                Type.SUBSYSTEM, null, systemUuid,
                "name", "AD5", "Sys-AD5", 3,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreate(structureElement);

        String description2 = "some other description";
        String comment2 = "some other comment";
        String description3 = "more description";
        String comment3 = "more comment";

        structureElement = new StructureElement(
                Type.SUBSYSTEM, null, systemUuid,
                "name", "AE1", "Sys-AE1", 3,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDescription(description2);
        structureElement.setComment(comment2);
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDescription(description3);
        structureElement.setComment(comment3);
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDeleted(Boolean.FALSE);
        responseStructureElement = ITUtilStructureElement.assertDeleteReject(structureElement);

        structureElement = new StructureElement(
                Type.SUBSYSTEM, null, systemUuid,
                "name", "AE2", "Sys-AE2", 3,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDescription(description2);
        structureElement.setComment(comment2);
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDescription(description3);
        structureElement.setComment(comment3);
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDeleted(Boolean.FALSE);
        responseStructureElement = ITUtilStructureElement.assertDeleteReject(structureElement);

        structureElement = new StructureElement(
                Type.SUBSYSTEM, null, systemUuid,
                "name", "AE3", "Sys-AE3", 3,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDescription(description2);
        structureElement.setComment(comment2);
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDescription(description3);
        structureElement.setComment(comment3);
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDeleted(Boolean.FALSE);
        responseStructureElement = ITUtilStructureElement.assertDeleteReject(structureElement);

        structureElement = new StructureElement(
                Type.SUBSYSTEM, null, systemUuid,
                "name", "AE4", "Sys-AE4", 3,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDescription(description2);
        structureElement.setComment(comment2);
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDescription(description3);
        structureElement.setComment(comment3);
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDeleted(Boolean.FALSE);
        responseStructureElement = ITUtilStructureElement.assertDeleteReject(structureElement);

        structureElement = new StructureElement(
                Type.SUBSYSTEM, null, systemUuid,
                "name", "AE5", "Sys-AE5", 3,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDescription(description2);
        structureElement.setComment(comment2);
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDescription(description3);
        structureElement.setComment(comment3);
        responseStructureElement = ITUtilStructureElement.assertUpdateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDeleted(Boolean.FALSE);
        responseStructureElement = ITUtilStructureElement.assertDeleteReject(structureElement);

        structureElement = new StructureElement(
                Type.SUBSYSTEM, null, systemUuid,
                "name", "AF1", "Sys-AF1", 3,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDeleted(Boolean.FALSE);
        responseStructureElement = ITUtilStructureElement.assertDeleteApprove(structureElement);

        structureElement = new StructureElement(
                Type.SUBSYSTEM, null, systemUuid,
                "name", "AF2", "Sys-AF2", 3,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDeleted(Boolean.FALSE);
        responseStructureElement = ITUtilStructureElement.assertDeleteApprove(structureElement);

        structureElement = new StructureElement(
                Type.SUBSYSTEM, null, systemUuid,
                "name", "AF3", "Sys-AF3", 3,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDeleted(Boolean.FALSE);
        responseStructureElement = ITUtilStructureElement.assertDeleteApprove(structureElement);

        structureElement = new StructureElement(
                Type.SUBSYSTEM, null, systemUuid,
                "name", "AF4", "Sys-AF4", 3,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDeleted(Boolean.FALSE);
        responseStructureElement = ITUtilStructureElement.assertDeleteApprove(structureElement);

        structureElement = new StructureElement(
                Type.SUBSYSTEM, null, systemUuid,
                "name", "AF5", "Sys-AF5", 3,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDeleted(Boolean.FALSE);
        responseStructureElement = ITUtilStructureElement.assertDeleteApprove(structureElement);

        structureElement = new StructureElement(
                Type.SUBSYSTEM, null, systemUuid,
                "name", "AG1", "Sys-AG1", 3,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDeleted(Boolean.FALSE);
        responseStructureElement = ITUtilStructureElement.assertDelete(structureElement);

        structureElement = new StructureElement(
                Type.SUBSYSTEM, null, systemUuid,
                "name", "AG2", "Sys-AG2", 3,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDeleted(Boolean.FALSE);
        responseStructureElement = ITUtilStructureElement.assertDelete(structureElement);

        structureElement = new StructureElement(
                Type.SUBSYSTEM, null, systemUuid,
                "name", "AG3", "Sys-AG3", 3,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDeleted(Boolean.FALSE);
        responseStructureElement = ITUtilStructureElement.assertDelete(structureElement);

        structureElement = new StructureElement(
                Type.SUBSYSTEM, null, systemUuid,
                "name", "AG4", "Sys-AG4", 3,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDeleted(Boolean.FALSE);
        responseStructureElement = ITUtilStructureElement.assertDelete(structureElement);

        structureElement = new StructureElement(
                Type.SUBSYSTEM, null, systemUuid,
                "name", "AG5", "Sys-AG5", 3,
                "description", Status.PENDING, Boolean.FALSE, Boolean.FALSE,
                null, "test who", "test comment");
        responseStructureElement = ITUtilStructureElement.assertCreateApprove(structureElement);
        structureElement = responseStructureElement;
        structureElement.setDeleted(Boolean.FALSE);
        responseStructureElement = ITUtilStructureElement.assertDelete(structureElement);

        // 60 subsystem entries

        try {
            // from first structure element
            assertNotNull(uuid);

            ITUtilStructureElement.assertRead("/SUBSYSTEM?queryFields=MNEMONIC&queryValues=A__",                                                  45, 45);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?statuses=PENDING&queryFields=MNEMONIC&queryValues=A__",                                 10, 10);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?statuses=APPROVED&queryFields=MNEMONIC&queryValues=A__",                                20, 20);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?statuses=CANCELLED&queryFields=MNEMONIC&queryValues=A__",                                5,  5);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?statuses=REJECTED&queryFields=MNEMONIC&queryValues=A__",                                10, 10);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?statuses=PENDING&statuses=APPROVED&queryFields=MNEMONIC&queryValues=A__",               30, 30);

            ITUtilStructureElement.assertRead("/SUBSYSTEM?deleted=false&queryFields=MNEMONIC&queryValues=A__",                                    30, 30);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?deleted=false&statuses=PENDING&queryFields=MNEMONIC&queryValues=A__",                    5,  5);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?deleted=false&statuses=APPROVED&queryFields=MNEMONIC&queryValues=A__",                  15, 15);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?deleted=false&statuses=CANCELLED&queryFields=MNEMONIC&queryValues=A__",                  5,  5);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?deleted=false&statuses=REJECTED&queryFields=MNEMONIC&queryValues=A__",                   5,  5);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?deleted=false&statuses=PENDING&statuses=APPROVED&queryFields=MNEMONIC&queryValues=A__", 20, 20);

            ITUtilStructureElement.assertRead("/SUBSYSTEM?deleted=true&queryFields=MNEMONIC&queryValues=A__",                                     15, 15);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?deleted=true&statuses=PENDING&queryFields=MNEMONIC&queryValues=A__",                     5,  5);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?deleted=true&statuses=APPROVED&queryFields=MNEMONIC&queryValues=A__",                    5,  5);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?deleted=true&statuses=CANCELLED&queryFields=MNEMONIC&queryValues=A__",                   0,  0);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?deleted=true&statuses=REJECTED&queryFields=MNEMONIC&queryValues=A__",                    5,  5);
            ITUtilStructureElement.assertRead("/SUBSYSTEM?deleted=true&statuses=PENDING&statuses=APPROVED&queryFields=MNEMONIC&queryValues=A__",  10, 10);

            ITUtilStructureElement.assertRead("/children/SUBSYSTEM/" + uuid.toString(),               0, 0);
            ITUtilStructureElement.assertRead("/children/SYSTEM/" + systemUuid.toString(),           -1, 1);
            ITUtilStructureElement.assertRead("/children/SYSTEMGROUP/" + systemGroupUuid.toString(), -1, 1);

            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sys-AA1", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sys-AA2", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sys-AA3", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sys-AA4", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sys-AA5", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sys-AB1", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sys-AB2", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sys-AB3", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sys-AB4", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sys-AB5", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sys-AC1", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sys-AC2", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sys-AC3", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sys-AC4", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sys-AC5", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sys-AD1", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sys-AD2", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sys-AD3", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sys-AD4", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sys-AD5", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sys-AE1", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sys-AE2", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sys-AE3", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sys-AE4", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sys-AE5", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sys-AF1", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sys-AF2", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sys-AF3", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sys-AF4", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sys-AF5", Boolean.FALSE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sys-AG1", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sys-AG2", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sys-AG3", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sys-AG4", Boolean.TRUE);
            ITUtilStructureElement.assertExists(Type.SUBSYSTEM, "Sys-AG5", Boolean.TRUE);
        } catch (Exception e) {
            fail();
        }
    }

}
